# -*- coding: utf-8 -*-
'''
pytestsalt.fixtures.daemons
~~~~~~~~~~~~~~~~~~~~~~~~~~~

Salt daemons fixtures
'''
# pylint: disable=redefined-outer-name

# Import python libs
from __future__ import absolute_import, print_function
import os
import sys
import logging
import subprocess

# Import 3rd-party libs
import pytest

from pytestsalt.utils import SaltCliScriptBase, SaltDaemonScriptBase, start_daemon

log = logging.getLogger(__name__)


@pytest.fixture
def salt_cli_default_timeout():
    '''
    Default timeout for CLI tools
    '''


@pytest.fixture(scope='session')
def salt_version(_cli_bin_dir, cli_master_script_name, python_executable_path):
    '''
    Return the salt version for the CLI install
    '''
    import salt  # pylint: disable=unused-import
    try:
        import salt.ext.six as six
    except ImportError:
        import six
    args = [
        os.path.join(_cli_bin_dir, cli_master_script_name),
        '--version'
    ]
    if sys.platform.startswith('win'):
        # We always need to prefix the call arguments with the python executable on windows
        args.insert(0, python_executable_path)

    proc = subprocess.Popen(args, stdout=subprocess.PIPE)
    stdout, stderr = proc.communicate()
    version = stdout.split()[1]
    if six.PY3:
        version = version.decode(__salt_system_encoding__)
    return version


@pytest.fixture
def salt_master_before_start():
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work before starting
    the salt-master and after ending it.
    '''
    # Prep routines go here

    # Start the salt-master
    yield

    # Clean routines go here


@pytest.fixture
def salt_master_after_start(salt_master):
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work after starting
    the salt-master and before ending it.
    '''
    # Prep routines go here

    # Resume test execution
    yield

    # Clean routines go here


@pytest.fixture
def salt_master(request,
                conf_dir,
                master_id,
                master_config,
                salt_master_before_start,  # pylint: disable=unused-argument
                log_server,  # pylint: disable=unused-argument
                master_log_prefix,
                cli_master_script_name,
                _cli_bin_dir,
                _salt_fail_hard):
    '''
    Returns a running salt-master
    '''
    return start_daemon(request,
                        daemon_name='salt-master',
                        daemon_id=master_id,
                        daemon_log_prefix=master_log_prefix,
                        daemon_cli_script_name=cli_master_script_name,
                        daemon_config=master_config,
                        daemon_config_dir=conf_dir,
                        daemon_class=SaltMaster,
                        bin_dir_path=_cli_bin_dir,
                        fail_hard=_salt_fail_hard,
                        event_listener_config_dir=conf_dir,
                        start_timeout=60)


@pytest.fixture(scope='session')
def session_salt_master_before_start():
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work before starting
    the salt-master and after ending it.
    '''
    # Prep routines go here

    # Start the salt-master
    yield

    # Clean routines go here


@pytest.fixture(scope='session')
def session_salt_master_after_start(session_salt_master):
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work after starting
    the salt-master and before ending it.
    '''
    # Prep routines go here

    # Resume test execution
    yield

    # Clean routines go here


@pytest.fixture(scope='session')
def session_salt_master(request,
                        session_conf_dir,
                        session_master_id,
                        session_master_config,
                        session_salt_master_before_start,  # pylint: disable=unused-argument
                        log_server,  # pylint: disable=unused-argument
                        session_master_log_prefix,
                        cli_master_script_name,
                        _cli_bin_dir,
                        _salt_fail_hard):
    '''
    Returns a running salt-master
    '''
    return start_daemon(request,
                        daemon_name='salt-master',
                        daemon_id=session_master_id,
                        daemon_log_prefix=session_master_log_prefix,
                        daemon_cli_script_name=cli_master_script_name,
                        daemon_config=session_master_config,
                        daemon_config_dir=session_conf_dir,
                        daemon_class=SaltMaster,
                        bin_dir_path=_cli_bin_dir,
                        fail_hard=_salt_fail_hard,
                        event_listener_config_dir=session_conf_dir,
                        start_timeout=60)


@pytest.fixture
def salt_master_of_masters_before_start():
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work before starting
    the salt-master and after ending it.
    '''
    # Prep routines go here

    # Start the salt-master
    yield

    # Clean routines go here


@pytest.fixture
def salt_master_of_masters_after_start(salt_master_of_masters):
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work after starting
    the salt-master and before ending it.
    '''
    # Prep routines go here

    # Resume test execution
    yield

    # Clean routines go here


@pytest.fixture
def salt_master_of_masters(request,
                           master_of_masters_conf_dir,
                           master_of_masters_id,
                           master_of_masters_config,
                           salt_master_of_masters_before_start,  # pylint: disable=unused-argument
                           log_server,  # pylint: disable=unused-argument
                           master_of_masters_log_prefix,
                           cli_master_script_name,
                           _cli_bin_dir,
                           _salt_fail_hard):
    '''
    Returns a running salt-master
    '''
    return start_daemon(request,
                        daemon_name='salt-master',
                        daemon_id=master_of_masters_id,
                        daemon_log_prefix=master_of_masters_log_prefix,
                        daemon_cli_script_name=cli_master_script_name,
                        daemon_config=master_of_masters_config,
                        daemon_config_dir=master_of_masters_conf_dir,
                        daemon_class=SaltMaster,
                        bin_dir_path=_cli_bin_dir,
                        fail_hard=_salt_fail_hard,
                        event_listener_config_dir=master_of_masters_conf_dir,
                        start_timeout=60)


@pytest.fixture(scope='session')
def session_salt_master_of_masters_before_start():
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work before starting
    the salt-master and after ending it.
    '''
    # Prep routines go here

    # Start the salt-master
    yield

    # Clean routines go here


@pytest.fixture(scope='session')
def session_salt_master_of_masters_after_start(session_salt_master_of_masters):
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work after starting
    the salt-master and before ending it.
    '''
    # Prep routines go here

    # Resume test execution
    yield

    # Clean routines go here


@pytest.fixture(scope='session')
def session_salt_master_of_masters(request,
                                   session_master_of_masters_conf_dir,
                                   session_master_of_masters_id,
                                   session_master_of_masters_config,
                                   session_salt_master_of_masters_before_start,  # pylint: disable=unused-argument
                                   log_server,  # pylint: disable=unused-argument
                                   session_master_of_masters_log_prefix,
                                   cli_master_script_name,
                                   _cli_bin_dir,
                                   _salt_fail_hard):
    '''
    Returns a running salt-master
    '''
    return start_daemon(request,
                        daemon_name='salt-master',
                        daemon_id=session_master_of_masters_id,
                        daemon_log_prefix=session_master_of_masters_log_prefix,
                        daemon_cli_script_name=cli_master_script_name,
                        daemon_config=session_master_of_masters_config,
                        daemon_config_dir=session_master_of_masters_conf_dir,
                        daemon_class=SaltMaster,
                        bin_dir_path=_cli_bin_dir,
                        fail_hard=_salt_fail_hard,
                        event_listener_config_dir=session_master_of_masters_conf_dir,
                        start_timeout=60)


@pytest.fixture
def salt_minion_before_start():
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work before starting
    the salt-minion and after ending it.
    '''
    # Prep routines go here

    # Start the salt-minion
    yield

    # Clean routines go here


@pytest.fixture
def salt_minion_after_start(salt_minion):
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work after starting
    the salt-minion and before ending it.
    '''
    # Prep routines go here

    # Resume test execution
    yield

    # Clean routines go here


@pytest.fixture
def salt_minion(request,
                salt_master,
                minion_id,
                minion_config,
                salt_minion_before_start,  # pylint: disable=unused-argument
                minion_log_prefix,
                cli_minion_script_name,
                log_server,
                _cli_bin_dir,
                _salt_fail_hard,
                conf_dir):  # pylint: disable=unused-argument
    '''
    Returns a running salt-minion
    '''
    return start_daemon(request,
                        daemon_name='salt-minion',
                        daemon_id=minion_id,
                        daemon_log_prefix=minion_log_prefix,
                        daemon_cli_script_name=cli_minion_script_name,
                        daemon_config=minion_config,
                        daemon_config_dir=conf_dir,
                        daemon_class=SaltMinion,
                        bin_dir_path=_cli_bin_dir,
                        fail_hard=_salt_fail_hard,
                        event_listener_config_dir=conf_dir,
                        start_timeout=60)


@pytest.fixture(scope='session')
def session_salt_minion_before_start():
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work before starting
    the salt-minion and after ending it.
    '''
    # Prep routines go here

    # Start the salt-minion
    yield

    # Clean routines go here


@pytest.fixture(scope='session')
def session_salt_minion_after_start(session_salt_minion):
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work after starting
    the salt-minion and before ending it.
    '''
    # Prep routines go here

    # Resume test execution
    yield

    # Clean routines go here


@pytest.fixture(scope='session')
def session_salt_minion(request,
                        session_salt_master,
                        session_minion_id,
                        session_minion_config,
                        session_salt_minion_before_start,  # pylint: disable=unused-argument
                        session_minion_log_prefix,
                        cli_minion_script_name,
                        log_server,
                        _cli_bin_dir,
                        session_conf_dir,
                        _salt_fail_hard):
    '''
    Returns a running salt-minion
    '''
    return start_daemon(request,
                        daemon_name='salt-minion',
                        daemon_id=session_minion_id,
                        daemon_log_prefix=session_minion_log_prefix,
                        daemon_cli_script_name=cli_minion_script_name,
                        daemon_config=session_minion_config,
                        daemon_config_dir=session_conf_dir,
                        daemon_class=SaltMinion,
                        bin_dir_path=_cli_bin_dir,
                        fail_hard=_salt_fail_hard,
                        event_listener_config_dir=session_conf_dir,
                        start_timeout=60)


@pytest.fixture
def secondary_salt_minion_before_start():
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work before starting
    the salt-minion and after ending it.
    '''
    # Prep routines go here

    # Start the salt-minion
    yield

    # Clean routines go here


@pytest.fixture
def secondary_salt_minion_after_start(secondary_salt_minion):
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work after starting
    the salt-minion and before ending it.
    '''
    # Prep routines go here

    # Resume test execution
    yield

    # Clean routines go here


@pytest.fixture
def secondary_salt_minion(request,
                          salt_master,
                          secondary_minion_id,
                          secondary_minion_config,
                          secondary_salt_minion_before_start,  # pylint: disable=unused-argument
                          secondary_minion_log_prefix,
                          cli_minion_script_name,
                          log_server,
                          _cli_bin_dir,
                          _salt_fail_hard,
                          conf_dir,
                          secondary_conf_dir):  # pylint: disable=unused-argument
    '''
    Returns a running salt-minion
    '''
    return start_daemon(request,
                        daemon_name='salt-minion',
                        daemon_id=secondary_minion_id,
                        daemon_log_prefix=secondary_minion_log_prefix,
                        daemon_cli_script_name=cli_minion_script_name,
                        daemon_config=secondary_minion_config,
                        daemon_config_dir=secondary_conf_dir,
                        daemon_class=SaltMinion,
                        bin_dir_path=_cli_bin_dir,
                        fail_hard=_salt_fail_hard,
                        event_listener_config_dir=conf_dir,
                        start_timeout=60)


@pytest.fixture(scope='session')
def session_secondary_salt_minion_before_start():
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work before starting
    the salt-minion and after ending it.
    '''
    # Prep routines go here

    # Start the salt-minion
    yield

    # Clean routines go here


@pytest.fixture(scope='session')
def session_secondary_salt_minion_after_start(session_secondary_salt_minion):
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work after starting
    the salt-minion and before ending it.
    '''
    # Prep routines go here

    # Resume test execution
    yield

    # Clean routines go here


@pytest.fixture(scope='session')
def session_secondary_salt_minion(request,
                                  session_salt_master,
                                  session_secondary_minion_id,
                                  session_secondary_minion_config,
                                  session_secondary_salt_minion_before_start,  # pylint: disable=unused-argument
                                  session_secondary_minion_log_prefix,
                                  cli_minion_script_name,
                                  log_server,
                                  _cli_bin_dir,
                                  _salt_fail_hard,
                                  session_conf_dir,
                                  session_secondary_conf_dir):
    '''
    Returns a running salt-minion
    '''
    return start_daemon(request,
                        daemon_name='salt-minion',
                        daemon_id=session_secondary_minion_id,
                        daemon_log_prefix=session_secondary_minion_log_prefix,
                        daemon_cli_script_name=cli_minion_script_name,
                        daemon_config=session_secondary_minion_config,
                        daemon_config_dir=session_secondary_conf_dir,
                        daemon_class=SaltMinion,
                        bin_dir_path=_cli_bin_dir,
                        fail_hard=_salt_fail_hard,
                        event_listener_config_dir=session_conf_dir,
                        start_timeout=60)


@pytest.fixture
def salt_syndic_before_start():
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work before starting
    the salt-syndic and after ending it.
    '''
    # Prep routines go here

    # Start the daemon
    yield

    # Clean routines go here


@pytest.fixture
def salt_syndic_after_start(salt_syndic):
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work after starting
    the salt-master and before ending it.
    '''
    # Prep routines go here

    # Resume test execution
    yield

    # Clean routines go here


@pytest.fixture
def salt_syndic(request,
                syndic_conf_dir,
                syndic_id,
                syndic_config,
                salt_syndic_before_start,  # pylint: disable=unused-argument
                log_server,  # pylint: disable=unused-argument
                syndic_log_prefix,
                cli_syndic_script_name,
                conf_dir,
                _cli_bin_dir,
                _salt_fail_hard):
    '''
    Returns a running salt-syndic
    '''
    return start_daemon(request,
                        daemon_name='salt-syndic',
                        daemon_id=syndic_id,
                        daemon_log_prefix=syndic_log_prefix,
                        daemon_cli_script_name=cli_syndic_script_name,
                        daemon_config=syndic_config,
                        daemon_config_dir=syndic_conf_dir,
                        daemon_class=SaltSyndic,
                        bin_dir_path=_cli_bin_dir,
                        fail_hard=_salt_fail_hard,
                        event_listener_config_dir=conf_dir,
                        start_timeout=60)


@pytest.fixture(scope='session')
def session_salt_syndic_before_start():
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work before starting
    the salt-syndic and after ending it.
    '''
    # Prep routines go here

    # Start the daemon
    yield

    # Clean routines go here


@pytest.fixture(scope='session')
def session_salt_syndic_after_start(session_salt_syndic):
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work after starting
    the salt-master and before ending it.
    '''
    # Prep routines go here

    # Resume test execution
    yield

    # Clean routines go here


@pytest.fixture(scope='session')
def session_salt_syndic(request,
                        session_syndic_conf_dir,
                        session_syndic_id,
                        session_syndic_config,
                        session_salt_syndic_before_start,  # pylint: disable=unused-argument
                        log_server,  # pylint: disable=unused-argument
                        session_syndic_log_prefix,
                        cli_syndic_script_name,
                        session_conf_dir,
                        _cli_bin_dir,
                        _salt_fail_hard):
    '''
    Returns a running salt-syndic
    '''
    return start_daemon(request,
                        daemon_name='salt-syndic',
                        daemon_id=session_syndic_id,
                        daemon_log_prefix=session_syndic_log_prefix,
                        daemon_cli_script_name=cli_syndic_script_name,
                        daemon_config=session_syndic_config,
                        daemon_config_dir=session_syndic_conf_dir,
                        daemon_class=SaltSyndic,
                        bin_dir_path=_cli_bin_dir,
                        fail_hard=_salt_fail_hard,
                        event_listener_config_dir=session_conf_dir,
                        start_timeout=60)


@pytest.fixture
def salt_proxy_before_start():
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work before starting
    the salt-proxy and after ending it.
    '''
    # Prep routines go here

    # Start the salt-proxy
    yield

    # Clean routines go here


@pytest.fixture
def salt_proxy_after_start(salt_proxy):
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work after starting
    the salt-proxy and before ending it.
    '''
    # Prep routines go here

    # Resume test execution
    yield

    # Clean routines go here


@pytest.fixture
def salt_proxy(request,
               salt_master,
               proxy_id,
               proxy_config,
               salt_proxy_before_start,  # pylint: disable=unused-argument
               proxy_log_prefix,
               cli_proxy_script_name,
               log_server,
               _cli_bin_dir,
               _salt_fail_hard,
               conf_dir):  # pylint: disable=unused-argument
    '''
    Returns a running salt-proxy
    '''
    return start_daemon(request,
                        daemon_name='salt-proxy',
                        daemon_id=proxy_id,
                        daemon_log_prefix=proxy_log_prefix,
                        daemon_cli_script_name=cli_proxy_script_name,
                        daemon_config=proxy_config,
                        daemon_config_dir=conf_dir,
                        daemon_class=SaltProxy,
                        bin_dir_path=_cli_bin_dir,
                        fail_hard=_salt_fail_hard,
                        event_listener_config_dir=conf_dir,
                        start_timeout=60)


@pytest.fixture(scope='session')
def session_salt_proxy_before_start():
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work before starting
    the salt-minion and after ending it.
    '''
    # Prep routines go here

    # Start the salt-proxy
    yield

    # Clean routines go here


@pytest.fixture(scope='session')
def session_salt_proxy_after_start(session_salt_proxy):
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work after starting
    the salt-proxy and before ending it.
    '''
    # Prep routines go here

    # Resume test execution
    yield

    # Clean routines go here


@pytest.fixture(scope='session')
def session_salt_proxy(request,
                       session_salt_master,
                       session_proxy_id,
                       session_proxy_config,
                       session_salt_proxy_before_start,  # pylint: disable=unused-argument
                       session_proxy_log_prefix,
                       cli_proxy_script_name,
                       log_server,
                       _cli_bin_dir,
                       session_conf_dir,
                       _salt_fail_hard):
    '''
    Returns a running salt-proxy
    '''
    return start_daemon(request,
                        daemon_name='salt-proxy',
                        daemon_id=session_proxy_id,
                        daemon_log_prefix=session_proxy_log_prefix,
                        daemon_cli_script_name=cli_proxy_script_name,
                        daemon_config=session_proxy_config,
                        daemon_config_dir=session_conf_dir,
                        daemon_class=SaltProxy,
                        bin_dir_path=_cli_bin_dir,
                        fail_hard=_salt_fail_hard,
                        event_listener_config_dir=session_conf_dir,
                        start_timeout=60)


@pytest.fixture
def salt_before_start():
    '''
    This fixture should be overridden if you need to do
    some preparation work before running salt-call and
    clean up after ending it.
    '''
    # Prep routines go here

    # Run!
    yield

    # Clean routines go here


@pytest.fixture
def salt_after_start(salt):
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work after starting
    the salt CLI script and before ending it.
    '''
    # Prep routines go here

    # Resume test execution
    yield

    # Clean routines go here


@pytest.fixture
def salt(request,
         salt_minion,
         minion_config,
         _cli_bin_dir,
         conf_dir,
         cli_salt_script_name,
         salt_cli_default_timeout,
         salt_before_start,  # pylint: disable=unused-argument
         log_server,         # pylint: disable=unused-argument
         salt_log_prefix):   # pylint: disable=unused-argument
    '''
    Returns a salt fixture
    '''
    salt = Salt(request,
                minion_config,
                conf_dir,
                _cli_bin_dir,
                salt_log_prefix,
                cli_script_name=cli_salt_script_name,
                default_timeout=salt_cli_default_timeout)
    yield salt


@pytest.fixture(scope='session')
def session_salt_before_start():
    '''
    This fixture should be overridden if you need to do
    some preparation work before running salt-call and
    clean up after ending it.
    '''
    # Prep routines go here

    # Run!
    yield

    # Clean routines go here


@pytest.fixture(scope='session')
def session_salt_after_start(session_salt):
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work after starting
    the salt CLI script and before ending it.
    '''
    # Prep routines go here

    # Resume test execution
    yield

    # Clean routines go here


@pytest.fixture(scope='session')
def session_salt(request,
                 session_salt_minion,
                 session_minion_config,
                 _cli_bin_dir,
                 session_conf_dir,
                 cli_salt_script_name,
                 salt_cli_default_timeout,
                 session_salt_before_start,  # pylint: disable=unused-argument
                 log_server,         # pylint: disable=unused-argument
                 session_salt_log_prefix):   # pylint: disable=unused-argument
    '''
    Returns a salt fixture
    '''
    salt = Salt(request,
                session_minion_config,
                session_conf_dir,
                _cli_bin_dir,
                session_salt_log_prefix,
                cli_script_name=cli_salt_script_name,
                default_timeout=salt_cli_default_timeout)
    yield salt


@pytest.fixture
def salt_call_before_start():
    '''
    This fixture should be overridden if you need to do
    some preparation work before running salt-call and
    clean up after ending it.
    '''
    # Prep routines go here

    # Run!
    yield

    # Clean routines go here


@pytest.fixture
def salt_call_after_start(salt_call):
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work after starting
    the salt-call and before ending it.
    '''
    # Prep routines go here

    # Resume test execution
    yield

    # Clean routines go here


@pytest.fixture
def salt_call(request,
              salt_minion,
              salt_call_before_start,
              salt_call_log_prefix,
              cli_call_script_name,
              minion_config,
              conf_dir,
              _cli_bin_dir,
              salt_cli_default_timeout,
              log_server):  # pylint: disable=unused-argument
    '''
    Returns a salt_call fixture
    '''
    salt_call = SaltCall(request,
                         minion_config,
                         conf_dir,
                         _cli_bin_dir,
                         salt_call_log_prefix,
                         cli_script_name=cli_call_script_name,
                         default_timeout=salt_cli_default_timeout)
    yield salt_call


@pytest.fixture(scope='session')
def session_salt_call_before_start():
    '''
    This fixture should be overridden if you need to do
    some preparation work before running salt-call and
    clean up after ending it.
    '''
    # Prep routines go here

    # Run!
    yield

    # Clean routines go here


@pytest.fixture(scope='session')
def session_salt_call_after_start(session_salt_call):
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work after starting
    the salt-call and before ending it.
    '''
    # Prep routines go here

    # Resume test execution
    yield

    # Clean routines go here


@pytest.fixture(scope='session')
def session_salt_call(request,
                      session_salt_minion,
                      session_salt_call_before_start,
                      session_salt_call_log_prefix,
                      cli_call_script_name,
                      session_minion_config,
                      session_conf_dir,
                      _cli_bin_dir,
                      salt_cli_default_timeout,
                      log_server):  # pylint: disable=unused-argument
    '''
    Returns a salt_call fixture
    '''
    salt_call = SaltCall(request,
                         session_minion_config,
                         session_conf_dir,
                         _cli_bin_dir,
                         session_salt_call_log_prefix,
                         cli_script_name=cli_call_script_name,
                         default_timeout=salt_cli_default_timeout)
    yield salt_call


@pytest.fixture
def salt_key_before_start():
    '''
    this fixture should be overridden if you need to do
    some preparation work before running salt-key and
    clean up after ending it.
    '''
    # prep routines go here

    # run!
    yield

    # clean routines go here


@pytest.fixture
def salt_key_after_start(salt_key):
    '''
    this fixture should be overridden if you need to do
    some preparation and clean up work after starting
    the salt-key and before ending it.
    '''
    # prep routines go here

    # resume test execution
    yield

    # clean routines go here


@pytest.fixture
def salt_key(request,
             salt_master,
             salt_key_before_start,
             salt_key_log_prefix,
             cli_key_script_name,
             master_config,
             conf_dir,
             _cli_bin_dir,
             salt_cli_default_timeout,
             log_server):  # pylint: disable=unused-argument
    '''
    returns a salt_key fixture
    '''
    salt_key = SaltKey(request,
                       master_config,
                       conf_dir,
                       _cli_bin_dir,
                       salt_key_log_prefix,
                       cli_script_name=cli_key_script_name,
                       default_timeout=salt_cli_default_timeout)
    yield salt_key


@pytest.fixture(scope='session')
def session_salt_key_before_start():
    '''
    this fixture should be overridden if you need to do
    some preparation work before running salt-key and
    clean up after ending it.
    '''
    # prep routines go here

    # run!
    yield

    # clean routines go here


@pytest.fixture(scope='session')
def session_salt_key_after_start(session_salt_key):
    '''
    this fixture should be overridden if you need to do
    some preparation and clean up work after starting
    the salt-key and before ending it.
    '''
    # prep routines go here

    # resume test execution
    yield

    # clean routines go here


@pytest.fixture(scope='session')
def session_salt_key(request,
                     session_salt_master,
                     session_salt_key_before_start,
                     session_salt_key_log_prefix,
                     cli_key_script_name,
                     session_master_config,
                     session_conf_dir,
                     _cli_bin_dir,
                     salt_cli_default_timeout,
                     log_server):  # pylint: disable=unused-argument
    '''
    returns a salt_key fixture
    '''
    salt_key = SaltKey(request,
                       session_master_config,
                       session_conf_dir,
                       _cli_bin_dir,
                       session_salt_key_log_prefix,
                       cli_script_name=cli_key_script_name,
                       default_timeout=salt_cli_default_timeout)
    yield salt_key


@pytest.fixture
def salt_run_before_start():
    '''
    This fixture should be overridden if you need to do
    some preparation work before running salt-run and
    clean up after ending it.
    '''
    # Prep routines go here

    # Run!
    yield

    # Clean routines go here


@pytest.fixture
def salt_run_after_start(salt_run):
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work after starting
    the salt-run and before ending it.
    '''
    # Prep routines go here

    # Resume test execution
    yield

    # Clean routines go here


@pytest.fixture
def salt_run(request,
             salt_master,
             salt_run_before_start,  # pylint: disable=unused-argument
             salt_run_log_prefix,
             cli_run_script_name,
             conf_dir,
             master_config,
             _cli_bin_dir,
             salt_cli_default_timeout,
             log_server):  # pylint: disable=unused-argument
    '''
    Returns a salt_run fixture
    '''
    salt_run = SaltRun(request,
                       master_config,
                       conf_dir,
                       _cli_bin_dir,
                       salt_run_log_prefix,
                       cli_script_name=cli_run_script_name,
                       default_timeout=salt_cli_default_timeout)
    yield salt_run


@pytest.fixture(scope='session')
def session_salt_run_before_start():
    '''
    This fixture should be overridden if you need to do
    some preparation work before running salt-run and
    clean up after ending it.
    '''
    # Prep routines go here

    # Run!
    yield

    # Clean routines go here


@pytest.fixture(scope='session')
def session_salt_run_after_start(session_salt_run):
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work after starting
    the salt-run and before ending it.
    '''
    # Prep routines go here

    # Resume test execution
    yield

    # Clean routines go here


@pytest.fixture(scope='session')
def session_salt_run(request,
                     session_salt_master,
                     session_salt_run_before_start,  # pylint: disable=unused-argument
                     session_salt_run_log_prefix,
                     cli_run_script_name,
                     session_conf_dir,
                     session_master_config,
                     _cli_bin_dir,
                     salt_cli_default_timeout,
                     log_server):  # pylint: disable=unused-argument
    '''
    Returns a salt_run fixture
    '''
    salt_run = SaltRun(request,
                       session_master_config,
                       session_conf_dir,
                       _cli_bin_dir,
                       session_salt_run_log_prefix,
                       cli_script_name=cli_run_script_name,
                       default_timeout=salt_cli_default_timeout)
    yield salt_run


@pytest.fixture
def salt_ssh_before_start():
    '''
    This fixture should be overridden if you need to do
    some preparation work before running salt-ssh and
    clean up after ending it.
    '''
    # Prep routines go here

    # Run!
    yield

    # Clean routines go here


@pytest.fixture
def salt_ssh_after_start():
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work after starting
    the salt-ssh and before ending it.
    '''
    # Prep routines go here

    # Resume test execution
    yield

    # Clean routines go here


@pytest.fixture
def salt_ssh(request,
             sshd_server,
             conf_dir,
             salt_ssh_before_start,  # pylint: disable=unused-argument
             salt_ssh_log_prefix,
             _cli_bin_dir,
             cli_ssh_script_name,
             roster_config,
             salt_cli_default_timeout,
             log_server):  # pylint: disable=unused-argument
    '''
    Returns a salt_ssh fixture
    '''
    salt_ssh = SaltSSH(request,
                       roster_config,
                       conf_dir,
                       _cli_bin_dir,
                       salt_ssh_log_prefix,
                       cli_script_name=cli_ssh_script_name,
                       default_timeout=salt_cli_default_timeout)
    yield salt_ssh


@pytest.fixture(scope='session')
def session_salt_ssh_before_start():
    '''
    This fixture should be overridden if you need to do
    some preparation work before running salt-ssh and
    clean up after ending it.
    '''
    # Prep routines go here

    # Run!
    yield

    # Clean routines go here


@pytest.fixture(scope='session')
def session_salt_ssh_after_start():
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work after starting
    the salt-ssh and before ending it.
    '''
    # Prep routines go here

    # Resume test execution
    yield

    # Clean routines go here


@pytest.fixture(scope='session')
def session_salt_ssh(request,
                     session_sshd_server,
                     session_conf_dir,
                     session_salt_ssh_before_start,  # pylint: disable=unused-argument
                     session_salt_ssh_log_prefix,
                     _cli_bin_dir,
                     cli_ssh_script_name,
                     session_roster_config,
                     salt_cli_default_timeout,
                     log_server):  # pylint: disable=unused-argument
    '''
    Returns a salt_ssh fixture
    '''
    salt_ssh = SaltSSH(request,
                       session_roster_config,
                       session_conf_dir,
                       _cli_bin_dir,
                       session_salt_ssh_log_prefix,
                       cli_script_name=cli_ssh_script_name,
                       default_timeout=salt_cli_default_timeout)
    yield salt_ssh


@pytest.fixture
def sshd_server_before_start():
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work before starting
    the sshd server and after ending it.
    '''
    # Prep routines go here

    # Start the sshd server
    yield

    # Clean routines go here


@pytest.fixture
def sshd_server_after_start(sshd_server):
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work after starting
    the sshd server and before ending it.
    '''
    # Prep routines go here

    # Resume test execution
    yield

    # Clean routines go here


@pytest.fixture
def sshd_server(request,
                write_sshd_config,  # pylint: disable=unused-argument
                sshd_server_before_start,  # pylint: disable=unused-argument
                sshd_server_log_prefix,
                sshd_port,
                sshd_config_dir,
                sshd_priv_dir,
                log_server):  # pylint: disable=unused-argument
    '''
    Returns a running sshd server
    '''
    log.info('[%s] Starting pytest sshd server at port %s', sshd_server_log_prefix, sshd_port)
    attempts = 0
    while attempts <= 3:  # pylint: disable=too-many-nested-blocks
        attempts += 1
        process = SSHD(request,
                       {'port': sshd_port},
                       sshd_config_dir,
                       None,  # bin_dir_path,
                       sshd_server_log_prefix,
                       cli_script_name='sshd')
        process.start()
        if process.is_alive():
            try:
                connectable = process.wait_until_running(timeout=10)
                if connectable is False:
                    connectable = process.wait_until_running(timeout=5)
                    if connectable is False:
                        process.terminate()
                        if attempts >= 3:
                            pytest.xfail(
                                'The pytest sshd server({}) has failed to confirm '
                                'running status after {} attempts'.format(sshd_port, attempts))
                        continue
            except Exception as exc:  # pylint: disable=broad-except
                log.exception('[%s] %s', sshd_server_log_prefix, exc, exc_info=True)
                process.terminate()
                if attempts >= 3:
                    pytest.xfail(str(exc))
                continue
            log.info(
                '[%s] The pytest sshd server(%s) is running and accepting commands '
                'after %d attempts',
                sshd_server_log_prefix,
                sshd_port,
                attempts
            )
            yield process
            break
        else:
            process.terminate()
            continue
    else:
        pytest.xfail(
            'The pytest sshd server({}) has failed to start after {} attempts'.format(
                sshd_port,
                attempts-1
            )
        )
    log.info('[%s] Stopping pytest sshd server(%s)', sshd_server_log_prefix, sshd_port)
    process.terminate()
    log.info('[%s] pytest sshd server(%s) stopped', sshd_server_log_prefix, sshd_port)


@pytest.fixture(scope='session')
def session_sshd_server_before_start():
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work before starting
    the sshd server and after ending it.
    '''
    # Prep routines go here

    # Start the sshd server
    yield

    # Clean routines go here


@pytest.fixture
def session_sshd_server_after_start(sshd_server):
    '''
    This fixture should be overridden if you need to do
    some preparation and clean up work after starting
    the sshd server and before ending it.
    '''
    # Prep routines go here

    # Resume test execution
    yield

    # Clean routines go here


@pytest.fixture(scope='session')
def session_sshd_server(request,
                        session_write_sshd_config,  # pylint: disable=unused-argument
                        session_sshd_server_before_start,  # pylint: disable=unused-argument
                        session_sshd_server_log_prefix,
                        session_sshd_port,
                        session_sshd_config_dir,
                        session_sshd_priv_dir,
                        log_server):  # pylint: disable=unused-argument
    '''
    Returns a running sshd server
    '''
    log.info('[%s] Starting pytest sshd server at port %s', session_sshd_server_log_prefix, session_sshd_port)
    attempts = 0
    while attempts <= 3:  # pylint: disable=too-many-nested-blocks
        attempts += 1
        process = SSHD(request,
                       {'port': session_sshd_port},
                       session_sshd_config_dir,
                       None,  # bin_dir_path,
                       session_sshd_server_log_prefix,
                       cli_script_name='sshd')
        process.start()
        if process.is_alive():
            try:
                connectable = process.wait_until_running(timeout=10)
                if connectable is False:
                    connectable = process.wait_until_running(timeout=5)
                    if connectable is False:
                        process.terminate()
                        if attempts >= 3:
                            pytest.xfail(
                                'The pytest sshd server({}) has failed to confirm '
                                'running status after {} attempts'.format(session_sshd_port, attempts))
                        continue
            except Exception as exc:  # pylint: disable=broad-except
                log.exception('[%s] %s', session_sshd_server_log_prefix, exc, exc_info=True)
                process.terminate()
                if attempts >= 3:
                    pytest.xfail(str(exc))
                continue
            log.info(
                '[%s] The pytest sshd server(%s) is running and accepting commands '
                'after %d attempts',
                session_sshd_server_log_prefix,
                session_sshd_port,
                attempts
            )
            yield process
            break
        else:
            process.terminate()
            continue
    else:
        pytest.xfail(
            'The pytest sshd server({}) has failed to start after {} attempts'.format(
                session_sshd_port,
                attempts-1
            )
        )
    log.info('[%s] Stopping pytest sshd server(%s)', session_sshd_server_log_prefix, session_sshd_port)
    process.terminate()
    log.info('[%s] pytest sshd server(%s) stopped', session_sshd_server_log_prefix, session_sshd_port)


class Salt(SaltCliScriptBase):
    '''
    Class which runs salt-call commands
    '''

    def get_minion_tgt(self, **kwargs):
        return kwargs.pop('minion_tgt', self.config['id'])

    def process_output(self, tgt, stdout, stderr, cli_cmd):  # pylint: disable=signature-differs
        if 'No minions matched the target. No command was sent, no jid was assigned.\n' in stdout:
            stdout = stdout.split('\n', 1)[1:][0]
        old_stdout = None
        if '--show-jid' in cli_cmd and stdout.startswith('jid: '):
            old_stdout = stdout
            stdout = stdout.split('\n', 1)[-1].strip()
        stdout, stderr, json_out = SaltCliScriptBase.process_output(self, tgt, stdout, stderr, cli_cmd)
        if old_stdout is not None:
            stdout = old_stdout
        if json_out:
            if not isinstance(json_out, dict):
                # A string was most likely loaded, not what we want.
                return stdout, stderr, None
            return stdout, stderr, json_out[tgt]
        return stdout, stderr, json_out


class SaltCall(SaltCliScriptBase):
    '''
    Class which runs salt-call commands
    '''

    def get_script_args(self):
        return ['--retcode-passthrough']


class SaltKey(SaltCliScriptBase):
    '''
    Class which runs salt-key commands
    '''


class SaltRun(SaltCliScriptBase):
    '''
    Class which runs salt-run commands
    '''

    def process_output(self, tgt, stdout, stderr, cli_cmd):  # pylint: disable=signature-differs
        if 'No minions matched the target. No command was sent, no jid was assigned.\n' in stdout:
            stdout = stdout.split('\n', 1)[1:][0]
        stdout, stderr, json_out = SaltCliScriptBase.process_output(self, tgt, stdout, stderr, cli_cmd)
        return stdout, stderr, json_out


class SaltSSH(SaltCliScriptBase):
    '''
    Class which runs salt-ssh commands
    '''

    def get_script_args(self):
        return [
            '-l', 'trace',
            '-w',
            '--rand-thin-dir',
            '--roster-file={}'.format(os.path.join(self.config_dir, 'roster')),
            '--ignore-host-keys',
        ]

    def get_minion_tgt(self, **kwargs):
        return 'localhost'

    def process_output(self, tgt, stdout, stderr, cli_cmd):  # pylint: disable=signature-differs
        stdout, stderr, json_out = SaltCliScriptBase.process_output(self, tgt, stdout, stderr, cli_cmd)
        if json_out:
            return stdout, stderr, json_out[tgt]
        return stdout, stderr, json_out


class SaltMinion(SaltDaemonScriptBase):
    '''
    Class which runs the salt-minion daemon
    '''

    def get_script_args(self):
        script_args = ['-l', 'quiet']
        if sys.platform.startswith('win') is False:
            script_args.append('--disable-keepalive')
        return script_args

    def get_check_events(self):
        if sys.platform.startswith('win'):
            return super(SaltMinion, self).get_check_events()
        return set(['salt/{}/{}/start'.format(self.config['__role'], self.config['id'])])

    def get_check_ports(self):
        if sys.platform.startswith('win'):
            return set([self.config['tcp_pub_port'],
                        self.config['tcp_pull_port']])
        return super(SaltMinion, self).get_check_ports()


class SaltProxy(SaltDaemonScriptBase):
    '''
    Class which runs the salt-proxy daemon
    '''

    def get_script_args(self):
        script_args = ['-l', 'quiet']

        script_args.extend(['--proxyid', self.config['id']])
        if sys.platform.startswith('win') is False:
            script_args.append('--disable-keepalive')
        return script_args

    def get_check_events(self):
        if sys.platform.startswith('win'):
            return super(SaltProxy, self).get_check_events()
        return set(['salt/{}/{}/start'.format(self.config['__role'], self.config['id'])])

    def get_check_ports(self):
        if sys.platform.startswith('win'):
            return set([self.config['tcp_pub_port'],
                        self.config['tcp_pull_port']])
        return super(SaltProxy, self).get_check_ports()


class SaltMaster(SaltDaemonScriptBase):
    '''
    Class which runs the salt-master daemon
    '''

    def get_script_args(self):
        return ['-l', 'quiet']

    def get_check_events(self):
        if sys.platform.startswith('win'):
            return super(SaltMaster, self).get_check_events()
        return set(['salt/{}/{}/start'.format(self.config['__role'], self.config['id'])])

    def get_check_ports(self):
        if sys.platform.startswith('win'):
            return set([self.config['ret_port'],
                        self.config['publish_port']])
        return super(SaltMaster, self).get_check_ports()


class SaltSyndic(SaltDaemonScriptBase):
    '''
    Class which runs the salt-syndic daemon
    '''

    def get_script_args(self):
        return ['-l', 'quiet']


class SSHD(SaltDaemonScriptBase):
    '''
    Class which runs an sshd daemon
    '''

    def get_script_path(self, script_name):
        '''
        Returns the path to the script to run
        '''
        import pytestsalt.utils.compat as compat
        sshd = compat.which(self.cli_script_name)
        if not sshd:
            pytest.skip('"sshd" not found')
        return sshd

    def get_base_script_args(self):
        return ['-D', '-f', os.path.join(self.config_dir, 'sshd_config')]

    def get_check_ports(self):
        return [self.config['port']]


@pytest.hookimpl(tryfirst=True)
def pytest_runtest_setup(item):
    '''
    Fixtures injection based on markers
    '''
    for fixture in ('salt_master', 'salt_minion', 'salt_call', 'salt', 'salt_key', 'salt_run'):
        if fixture in item.fixturenames:
            after_start_fixture = '{}_after_start'.format(fixture)
            if after_start_fixture not in item.fixturenames:
                item.fixturenames.append(after_start_fixture)



# -*- coding: utf-8 -*-

# Import python libs
import logging

if not hasattr(logging, 'TRACE'):
    logging.TRACE = 5
    logging.addLevelName(logging.TRACE, 'TRACE')
if not hasattr(logging, 'GARBAGE'):
    logging.GARBAGE = 1
    logging.addLevelName(logging.GARBAGE, 'GARBAGE')


pytest_plugins = 'pytester', 'tornado'


def pytest_configure(config):
    config._inicache['log_format'] = '%(asctime)s,%(msecs)04.0f [%(name)-5s:%(lineno)-4d][%(processName)-8s][%(levelname)-8s] %(message)s'



# -*- coding: utf-8 -*-
'''
    pytestsalt.fixtures.stats
    ~~~~~~~~~~~~~~~~~~~~~~~~~

    Test session process stats fixtures
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import sys
from collections import OrderedDict

# Import 3rd-party libs
import psutil

# Import pytest libs
import pytest
from _pytest.terminal import TerminalReporter

IS_WINDOWS = sys.platform.startswith('win')


class SaltTerminalReporter(TerminalReporter):
    def __init__(self, config):
        TerminalReporter.__init__(self, config)
        self._session = None
        self._show_sys_stats = config.getoption('--sys-stats') is True
        self._sys_stats_no_children = config.getoption('--sys-stats-no-children') is True
        if config.getoption('--sys-stats-uss-mem') is True:
            self._sys_stats_mem_type = 'uss'
        else:
            self._sys_stats_mem_type = 'rss'

    @pytest.hookimpl(trylast=True)
    def pytest_sessionstart(self, session):
        TerminalReporter.pytest_sessionstart(self, session)
        self._session = session

    def pytest_runtest_logreport(self, report):
        TerminalReporter.pytest_runtest_logreport(self, report)
        if self.verbosity <= 0:
            return

        if report.when != 'call':
            return

        if self._show_sys_stats is False:
            return

        if self.verbosity > 1:
            self.ensure_newline()
            self.section('Processes Statistics', sep='-', bold=True)
            left_padding = len(max(['System'] + list(self._session.stats_processes), key=len))
            template = '  ...{dots}  {name}  -'
            if not IS_WINDOWS:
                template += '  SWAP: {swap:6.2f} %   CPU: {cpu:6.2f} %'
            else:
                template += '  CPU: {cpu:6.2f} %'
            template += '   MEM: {mem:6.2f} (Virtual Memory) %\n'
            stats = {
                'name': 'System',
                'dots': '.' * (left_padding - len('System')),
                'cpu': psutil.cpu_percent(),
                'mem': psutil.virtual_memory().percent
            }
            if not IS_WINDOWS:
                stats['swap'] = psutil.swap_memory().percent
            self.write(template.format(**stats))

            template = '  ...{dots}  {name}  -'
            if not IS_WINDOWS:
                template += '  SWAP: {swap:6.2f} %   CPU: {cpu:6.2f} %'
            else:
                template += '  CPU: {cpu:6.2f} %'
            template += '   MEM: {mem:6.2f} % ({m_type})'
            children_template = template + '   MEM SUM: {c_mem} % ({m_type})   CHILD PROCS: {c_count}\n'
            no_children_template = template + '\n'

            for name, psproc in self._session.stats_processes.items():
                template = no_children_template
                dots = '.' * (left_padding - len(name))
                pids = []
                try:
                    with psproc.oneshot():
                        stats = {
                            'name': name,
                            'dots': dots,
                            'cpu': psproc.cpu_percent(),
                            'mem': psproc.memory_percent(self._sys_stats_mem_type),
                            'm_type': self._sys_stats_mem_type.upper()
                        }
                        if self._sys_stats_no_children is False:
                            pids.append(psproc.pid)
                            children = psproc.children(recursive=True)
                            if children:
                                template = children_template
                                stats['c_count'] = 0
                                c_mem = stats['mem']
                                for child in children:
                                    if child.pid in pids:
                                        continue
                                    pids.append(child.pid)
                                    if not psutil.pid_exists(child.pid):
                                        continue
                                    try:
                                        c_mem += child.memory_percent(self._sys_stats_mem_type)
                                        stats['c_count'] += 1
                                    except psutil.NoSuchProcess:
                                        continue
                                if stats['c_count']:
                                    stats['c_mem'] = '{:6.2f}'.format(c_mem)
                                else:
                                    template = no_children_template
                        if not IS_WINDOWS:
                            stats['swap'] = psproc.memory_percent('swap')
                        self.write(template.format(**stats))
                except psutil.NoSuchProcess:
                    continue

    def _get_progress_information_message(self):
        msg = TerminalReporter._get_progress_information_message(self)
        if self.verbosity <= 0:
            return msg
        if self.config.getoption('--sys-stats') is False:
            return msg
        if self.verbosity == 1:
            msg = ' [CPU:{}%] [MEM:{}%]{}'.format(psutil.cpu_percent(),
                                                  psutil.virtual_memory().percent,
                                                  msg)
        return msg


def pytest_sessionstart(session):
    session.stats_processes = OrderedDict((
        ('Test Suite Run', psutil.Process(os.getpid())),
    ))


def pytest_addoption(parser):
    '''
    register argparse-style options and ini-style config values.
    '''
    output_options_group = parser.getgroup('Output Options')
    output_options_group.addoption(
        '--sys-stats',
        default=False,
        action='store_true',
        help='Print System CPU and MEM statistics after each test execution.'
    )
    output_options_group.addoption(
        '--sys-stats-no-children',
        default=False,
        action='store_true',
        help='Don\'t include child processes memory statistics.'
    )
    output_options_group.addoption(
        '--sys-stats-uss-mem',
        default=False,
        action='store_true',
        help='Use the USS("Unique Set Size", memory unique to a process which would be freed if the process was '
             'terminated) memory instead which is more expensive to calculate.'
    )


@pytest.mark.trylast
def pytest_configure(config):
    '''
    called after command line options have been parsed
    and all plugins and initial conftest files been loaded.
    '''
    # Register our terminal reporter
    if not getattr(config, 'slaveinput', None):
        standard_reporter = config.pluginmanager.getplugin('terminalreporter')
        salt_reporter = SaltTerminalReporter(standard_reporter.config)

        config.pluginmanager.unregister(standard_reporter)
        config.pluginmanager.register(salt_reporter, 'terminalreporter')



# -*- coding: utf-8 -*-
'''
pytestsalt.engines.pytest_engine
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Simple salt engine which will setup a socket to accept connections allowing us to know
when a daemon is up and running
'''

# Import python libs
from __future__ import absolute_import, print_function
import os
import sys
import errno
import socket
import logging

# Import salt libs
import salt.utils.event
try:
    import salt.utils.asynchronous
    HAS_SALT_ASYNC = True
except ImportError:
    HAS_SALT_ASYNC = False

# Import 3rd-party libs
from tornado import gen
from tornado import ioloop
from tornado import iostream
from tornado import netutil

log = logging.getLogger(__name__)

__virtualname__ = 'pytest'


def __virtual__():
    return 'pytest_engine_port' in __opts__  # pylint: disable=undefined-variable


def start():
    pytest_engine = PyTestEngine(__opts__)  # pylint: disable=undefined-variable
    pytest_engine.start()


class PyTestEngine(object):
    def __init__(self, opts):
        self.opts = opts
        self.id = opts['id']
        self.role = opts['__role']
        self.sock_dir = opts['sock_dir']
        self.port = int(opts['pytest_engine_port'])
        self.tcp_server_sock = None
        self.stop_sending_events_file = opts.get('pytest_stop_sending_events_file')

    def start(self):
        self.io_loop = ioloop.IOLoop()
        self.io_loop.make_current()
        self.io_loop.add_callback(self._start)
        self.io_loop.start()

    @gen.coroutine
    def _start(self):
        log.info('Starting Pytest Engine(role=%s, id=%s) on port %s', self.role, self.id, self.port)

        self.tcp_server_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.tcp_server_sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.tcp_server_sock.setblocking(0)
        # bind the socket to localhost on the config provided port
        self.tcp_server_sock.bind(('localhost', self.port))
        # become a server socket
        self.tcp_server_sock.listen(5)
        if HAS_SALT_ASYNC:
            with salt.utils.asynchronous.current_ioloop(self.io_loop):
                netutil.add_accept_handler(self.tcp_server_sock, self.handle_connection)
        else:
            netutil.add_accept_handler(self.tcp_server_sock, self.handle_connection)

        if self.role == 'master':
            yield self.fire_master_started_event()

    def handle_connection(self, connection, address):
        log.warning('Accepted connection from %s. Role: %s  ID: %s', address, self.role, self.id)
        # We just need to know that the daemon running the engine is alive...
        try:
            connection.shutdown(socket.SHUT_RDWR)  # pylint: disable=no-member
            connection.close()
        except socket.error as exc:
            if not sys.platform.startswith('darwin'):
                raise
            try:
                if exc.errno != errno.ENOTCONN:
                    raise
            except AttributeError:
                # This is not macOS !?
                pass

    @gen.coroutine
    def fire_master_started_event(self):
        master_start_event_tag = 'salt/master/{}/start'.format(self.id)
        log.info('Firing salt-%s started event. Tag: %s', self.role, master_start_event_tag)
        event_bus = salt.utils.event.get_master_event(self.opts, self.sock_dir, listen=False)
        load = {'id': self.id, 'tag': master_start_event_tag, 'data': {}}
        # 30 seconds should be more than enough to fire these events every second in order
        # for pytest-salt to pickup that the master is running
        timeout = 30
        while True:
            if self.stop_sending_events_file and not os.path.exists(self.stop_sending_events_file):
                log.info('The stop sending events file "marker" is done. Stop sending events...')
                break
            timeout -= 1
            try:
                event_bus.fire_event(load, master_start_event_tag, timeout=500)
                if timeout <= 0:
                    break
                yield gen.sleep(1)
            except iostream.StreamClosedError:
                break



# -*- coding: utf-8 -*-
'''
    pytestsalt.fixtures.log_server_tornado
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Tornado Log Server Fixture
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import logging
import threading

# Import 3rd-party libs
import msgpack
from tornado import gen
from tornado.ioloop import IOLoop
from tornado.tcpserver import TCPServer
from tornado.iostream import StreamClosedError

log = logging.getLogger(__name__)


class LogServer(TCPServer):

    @gen.coroutine
    def handle_stream(self, stream, address):
        unpacker = msgpack.Unpacker(raw=False)
        while True:
            try:
                wire_bytes = yield stream.read_bytes(1024, partial=True)
                if not wire_bytes:
                    break
                try:
                    unpacker.feed(wire_bytes)
                except msgpack.exceptions.BufferFull:
                    # Start over loosing some data?!
                    unpacker = msgpack.Unpacker(raw=False)
                    unpacker.feed(wire_bytes)
                for record_dict in unpacker:
                    record = logging.makeLogRecord(record_dict)
                    logger = logging.getLogger(record.name)
                    logger.handle(record)
            except (EOFError, KeyboardInterrupt, SystemExit, StreamClosedError):
                break
            except Exception as exc:  # pylint: disable=broad-except
                log.exception(exc)


def log_server_tornado(log_server_port):
    '''
    Starts a log server.
    '''

    def process_logs(port):
        server = LogServer()
        server.listen(port, address='127.0.0.1')
        try:
            IOLoop.current().start()
        except KeyboardInterrupt:
            pass

        server.stop()

    process_queue_thread = threading.Thread(target=process_logs, args=(log_server_port,))
    process_queue_thread.daemon = True
    process_queue_thread.start()



# -*- coding: utf-8 -*-
'''
pytestsalt.fixtures.log
~~~~~~~~~~~~~~~~~~~~~~~

Log server fixture which creates a server that receives log records
from external process to log them in the current process
'''

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
#import sys
import logging

# Import pytest libs
import pytest

#if sys.version_info > (3, 5):
#    from pytestsalt.utils.log_server_asyncio import log_server_asyncio as salt_log_server
#else:
#    from pytestsalt.utils.log_server_tornado import log_server_tornado as salt_log_server
from pytestsalt.utils.log_server_tornado import log_server_tornado as salt_log_server


log = logging.getLogger(__name__)


@pytest.fixture(scope='session')
def log_server_level(request):
    # If PyTest has no logging configured, default to ERROR level
    levels = [logging.ERROR]
    logging_plugin = request.config.pluginmanager.get_plugin('logging-plugin')
    try:
        level = logging_plugin.log_cli_handler.level
        if level is not None:
            levels.append(level)
    except AttributeError:
        # PyTest CLI logging not configured
        pass
    try:
        level = logging_plugin.log_file_level
        if level is not None:
            levels.append(level)
    except AttributeError:
        # PyTest Log File logging not configured
        pass

    level_str = logging.getLevelName(min(levels))
    return level_str


@pytest.fixture(scope='session')
def log_server(salt_log_port):
    log.info('Starting log server')
    salt_log_server(salt_log_port)
    log.info('Log Server Started')
    # Run tests
    yield



# -*- coding: utf-8 -*-
'''
pytestsalt.salt.log_handlers.pytest_log_handler
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Salt External Logging Handler
'''

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import socket
import threading
import logging
from multiprocessing import Queue

# Import 3rd-party libs
import msgpack

# Import Salt libs
import salt.log.setup
# pylint: disable=no-member,invalid-name
try:
    import salt.utils.stringutils
    to_unicode = salt.utils.stringutils.to_unicode
except ImportError:
    import salt.utils
    to_unicode = salt.utils.to_unicode
try:
    import salt.utils.platform
    is_darwin = salt.utils.platform.is_darwin
except ImportError:
    import salt.utils
    is_darwin = salt.utils.is_darwin
# pylint: enable=no-member,invalid-name

__virtualname__ = 'pytest_log_handler'

log = logging.getLogger(__name__)


def __virtual__():
    if 'pytest_log_port' not in __opts__:
        return False, "'pytest_log_port' not in options"
    return True


def setup_handlers():
    host_addr = __opts__.get('pytest_log_host')
    if not host_addr:
        import subprocess
        if __opts__['pytest_windows_guest'] is True:
            proc = subprocess.Popen('ipconfig', stdout=subprocess.PIPE)
            for line in proc.stdout.read().strip().encode(__salt_system_encoding__).splitlines():
                if 'Default Gateway' in line:
                    parts = line.split()
                    host_addr = parts[-1]
                    break
        else:
            proc = subprocess.Popen(
                "netstat -rn | grep -E '^0.0.0.0|default' | awk '{ print $2 }'",
                shell=True, stdout=subprocess.PIPE
            )
            host_addr = proc.stdout.read().strip().encode(__salt_system_encoding__)
    host_port = __opts__['pytest_log_port']
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        sock.connect((host_addr, host_port))
    except socket.error as exc:
        # Don't even bother if we can't connect
        log.warning('Cannot connect back to log server: %s', exc)
        return
    finally:
        sock.close()

    if is_darwin():
        # The maximum for the multiprocessing queue on MacOS is 32767, so if we running on MacOS
        # then we use that maximum.
        queue_size = 32767
    else:
        # One million log messages is more than enough to queue.
        # Above that value, if `process_queue` can't process fast enough,
        # start dropping. This will contain a memory leak in case `process_queue`
        # can't process fast enough of in case it can't deliver the log records at all.
        queue_size = 10000000
    queue = Queue(queue_size)
    handler = salt.log.setup.SaltLogQueueHandler(queue)
    level = salt.log.setup.LOG_LEVELS[(__opts__.get('pytest_log_level') or 'error').lower()]
    handler.setLevel(level)
    pytest_log_prefix = os.environ.get('PYTEST_LOG_PREFIX') or __opts__['pytest_log_prefix']
    process_queue_thread = threading.Thread(target=process_queue,
                                            args=(host_addr,
                                                  host_port,
                                                  pytest_log_prefix,
                                                  queue))
    process_queue_thread.daemon = True
    process_queue_thread.start()
    return handler


def process_queue(host, port, prefix, queue):
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        sock.connect((host, port))
    except socket.error:
        sock.close()
        return

    log.warning('Sending log records to Remote log server')
    while True:
        try:
            record = queue.get()
            if record is None:
                # A sentinel to stop processing the queue
                break
            # Just send every log. Filtering will happen on the main process
            # logging handlers
            record_dict = record.__dict__
            record_dict['msg'] = '[{}] {}'.format(to_unicode(prefix), to_unicode(record_dict['msg']))
            sock.sendall(msgpack.dumps(record_dict, encoding='utf-8'))
        except (IOError, EOFError, KeyboardInterrupt, SystemExit):
            break
        except Exception as exc:  # pylint: disable=broad-except
            log.warning(
                'An exception occurred in the pytest salt logging '
                'queue thread: %s',
                exc,
                exc_info_on_loglevel=logging.DEBUG
            )



# -*- coding: utf-8 -*-
'''
pytestsalt.fixtures.config
~~~~~~~~~~~~~~~~~~~~~~~~~~

pytest salt configuration related fixtures
'''
# pylint: disable=redefined-outer-name,too-many-arguments,too-many-locals

# Import python libs
from __future__ import absolute_import, print_function
import os
import sys
import copy
import pprint
import logging
import subprocess

# Import 3rd-party libs
import pytest

IS_WINDOWS = sys.platform.startswith('win')

if IS_WINDOWS:
    import win32api  # pylint: disable=import-error
else:
    import pwd

DEFAULT_MOM_ID = 'pytest-salt-mom'
DEFAULT_MASTER_ID = 'pytest-salt-master'
DEFAULT_MINION_ID = 'pytest-salt-minion'
DEFAULT_SYNDIC_ID = 'pytest-salt-syndic'
DEFAULT_SECONDARY_MINION_ID = 'pytest-salt-sec-minion'
DEFAULT_PROXY_MINION_ID = 'pytest-salt-proxy'
DEFAULT_SESSION_MOM_ID = 'pytest-session-salt-mom'
DEFAULT_SESSION_MASTER_ID = 'pytest-session-salt-master'
DEFAULT_SESSION_MINION_ID = 'pytest-session-salt-minion'
DEFAULT_SESSION_SYNDIC_ID = 'pytest-session-salt-syndic'
DEFAULT_SESSION_SECONDARY_MINION_ID = 'pytest-session-salt-sec-minion'
DEFAULT_SESSION_PROXY_MINION_ID = 'pytest-session-salt-proxy'

log = logging.getLogger(__name__)


class Counter(object):  # pylint: disable=too-few-public-methods
    '''
    Simple counter class which increases count on every call to it's instance
    '''
    def __init__(self):
        self.counter = 0

    def __call__(self):
        try:
            return self.counter
        finally:
            self.counter += 1


def pytest_addoption(parser):
    '''
    Add pytest salt plugin daemons related options
    '''
    saltparser = parser.getgroup('Salt Plugin Options')
    saltparser.addoption(
        '--cli-bin-dir',
        default=None,
        help=('Path to the bin directory where the salt CLI scripts can be '
              'found. Defaults to the directory name of the python executable '
              'running py.test')
    )
    saltparser.addoption(
        '--salt-fail-hard',
        default=None,
        action='store_true',
        help=('If a salt daemon fails to start, the test is marked as XFailed. '
              'If this flag is passed, then a test failure is raised instead of XFail.')
    )
    parser.addini(
        'cli_bin_dir',
        default=None,
        help=('Path to the bin directory where the salt CLI scripts can be '
              'found. Defaults to the directory name of the python executable '
              'running py.test')
    )
    parser.addini(
        'salt_fail_hard',
        default=None,
        type='bool',
        help=('If a salt daemon fails to start, the test is marked as XFailed. '
              'If this flag is set, then a test failure is raised instead of XFail.')
    )


@pytest.hookimpl(trylast=True)
def pytest_report_header(config, startdir):
    '''
    return a string to be displayed as header info for terminal reporting.
    '''
    # Store a reference to where the base directory of the project is
    config.startdir = startdir


@pytest.fixture(scope='session')
def python_executable_path():
    '''
    return the python executable path
    '''
    return sys.executable


@pytest.fixture(scope='session')
def cli_bin_dir(request, python_executable_path):
    '''
    Return the path to the CLI script directory to use
    '''
    # Default to the directory of the current python executable
    return os.path.dirname(python_executable_path)


@pytest.fixture(scope='session')
def _cli_bin_dir(request, cli_bin_dir):
    '''
    Return the path to the CLI script directory to use
    '''
    path = request.config.getoption('cli_bin_dir')
    if path is not None:
        # We were passed --cli-bin-dir as a CLI option
        return os.path.expanduser(path)

    # The path was not passed as a CLI option
    path = request.config.getini('cli_bin_dir')
    if path:
        # We were passed cli_bin_dir as a INI option
        return os.path.expanduser(path)

    return cli_bin_dir


@pytest.fixture(scope='session')
def salt_fail_hard(request):
    '''
    Return the salt fail hard value
    '''
    return False


@pytest.fixture(scope='session')
def _salt_fail_hard(request, salt_fail_hard):
    '''
    Return the salt fail hard value
    '''
    fail_hard = request.config.getoption('salt_fail_hard')
    if fail_hard is not None:
        # We were passed --salt-fail-hard as a CLI option
        return fail_hard

    # The salt fail hard was not passed as a CLI option
    fail_hard = request.config.getini('salt_fail_hard')
    if fail_hard != []:
        # We were passed salt_fail_hard as a INI option
        return fail_hard

    return salt_fail_hard


@pytest.fixture(scope='session')
def running_username():
    '''
    Returns the current username
    '''
    if IS_WINDOWS:
        return win32api.GetUserName()
    return pwd.getpwuid(os.getuid()).pw_name


@pytest.fixture(scope='session')
def salt_master_id_counter():
    '''
    Fixture which return a number to include in the master ID.
    Every call to this fixture increases the counter.
    '''
    return Counter()


@pytest.fixture(scope='session')
def salt_master_of_masters_id_counter():
    '''
    Fixture which return a number to include in the master ID.
    Every call to this fixture increases the counter.
    '''
    return Counter()


@pytest.fixture(scope='session')
def salt_minion_id_counter():
    '''
    Fixture which return a number to include in the minion ID.
    Every call to this fixture increases the counter.
    '''
    return Counter()


@pytest.fixture(scope='session')
def salt_syndic_id_counter():
    '''
    Fixture which return a number to include in the syndic ID.
    Every call to this fixture increases the counter.
    '''
    return Counter()


@pytest.fixture(scope='session')
def salt_proxy_id_counter():
    '''
    Fixture which return a number to include in the minion proxy ID.
    Every call to this fixture increases the counter.
    '''
    return Counter()


@pytest.fixture(scope='session')
def cli_master_script_name():
    '''
    Return the CLI script basename
    '''
    return 'salt-master'


@pytest.fixture(scope='session')
def cli_minion_script_name():
    '''
    Return the CLI script basename
    '''
    return 'salt-minion'


@pytest.fixture(scope='session')
def cli_proxy_script_name():
    '''
    Return the CLI script basename
    '''
    return 'salt-proxy'


@pytest.fixture(scope='session')
def cli_salt_script_name():
    '''
    Return the CLI script basename
    '''
    return 'salt'


@pytest.fixture(scope='session')
def cli_run_script_name():
    '''
    Return the CLI script basename
    '''
    return 'salt-run'


@pytest.fixture(scope='session')
def cli_key_script_name():
    '''
    Return the CLI script basename
    '''
    return 'salt-key'


@pytest.fixture(scope='session')
def cli_call_script_name():
    '''
    Return the CLI script basename
    '''
    return 'salt-call'


@pytest.fixture(scope='session')
def cli_syndic_script_name():
    '''
    Return the CLI script basename
    '''
    return 'salt-syndic'


@pytest.fixture(scope='session')
def cli_ssh_script_name():
    '''
    Return the CLI script basename
    '''
    return 'salt-ssh'


@pytest.fixture
def master_of_masters_id(salt_master_of_masters_id_counter):
    '''
    Returns the master of masters id
    '''
    return DEFAULT_MOM_ID + '-{}'.format(salt_master_of_masters_id_counter())


@pytest.fixture
def master_id(salt_master_id_counter):
    '''
    Returns the master id
    '''
    return DEFAULT_MASTER_ID + '-{}'.format(salt_master_id_counter())


@pytest.fixture
def minion_id(salt_minion_id_counter):
    '''
    Returns the minion id
    '''
    return DEFAULT_MINION_ID + '-{}'.format(salt_minion_id_counter())


@pytest.fixture
def secondary_minion_id(salt_minion_id_counter):
    '''
    Returns the secondary minion id
    '''
    return DEFAULT_SECONDARY_MINION_ID + '-{}'.format(salt_minion_id_counter())


@pytest.fixture
def syndic_id(salt_syndic_id_counter):
    '''
    Returns the syndic id
    '''
    return DEFAULT_SYNDIC_ID + '-{}'.format(salt_syndic_id_counter())


@pytest.fixture
def proxy_id(salt_proxy_id_counter):
    '''
    Returns the proxy minion id
    '''
    return DEFAULT_PROXY_MINION_ID + '-{}'.format(salt_proxy_id_counter())


@pytest.fixture(scope='session')
def session_master_of_masters_id(salt_master_of_masters_id_counter):
    '''
    Returns the master of masters id
    '''
    return DEFAULT_SESSION_MOM_ID + '-{}'.format(salt_master_of_masters_id_counter())


@pytest.fixture(scope='session')
def session_master_id(salt_master_id_counter):
    '''
    Returns the session scoped master id
    '''
    return DEFAULT_SESSION_MASTER_ID + '-{}'.format(salt_master_id_counter())


@pytest.fixture(scope='session')
def session_minion_id(salt_minion_id_counter):
    '''
    Returns the session scoped minion id
    '''
    return DEFAULT_SESSION_MINION_ID + '-{}'.format(salt_minion_id_counter())


@pytest.fixture(scope='session')
def session_secondary_minion_id(salt_minion_id_counter):
    '''
    Returns the session scoped secondary minion id
    '''
    return DEFAULT_SESSION_SECONDARY_MINION_ID + '-{}'.format(salt_minion_id_counter())


@pytest.fixture(scope='session')
def session_syndic_id(salt_syndic_id_counter):
    '''
    Returns the session scoped syndic id
    '''
    return DEFAULT_SESSION_SYNDIC_ID + '-{}'.format(salt_syndic_id_counter())


@pytest.fixture(scope='session')
def session_proxy_id(salt_proxy_id_counter):
    '''
    Returns the session scoped minion id
    '''
    return DEFAULT_SESSION_PROXY_MINION_ID + '-{}'.format(salt_proxy_id_counter())


@pytest.fixture
def master_config_file(conf_dir):
    '''
    Returns the path to the salt master configuration file
    '''
    return conf_dir.join('master').realpath().strpath


@pytest.fixture
def master_of_masters_config_file(master_of_masters_conf_dir):
    '''
    Returns the path to the salt master configuration file
    '''
    return master_of_masters_conf_dir.join('master').realpath().strpath


@pytest.fixture
def minion_config_file(conf_dir):
    '''
    Returns the path to the salt minion configuration file
    '''
    return conf_dir.join('minion').realpath().strpath


@pytest.fixture
def secondary_minion_config_file(secondary_conf_dir):
    '''
    Returns the path to the salt secondary minion configuration file
    '''
    return secondary_conf_dir.join('minion').realpath().strpath


@pytest.fixture
def proxy_config_file(conf_dir):
    '''
    Returns the path to the salt minion configuration file
    '''
    return conf_dir.join('proxy').realpath().strpath


@pytest.fixture
def master_config_overrides():
    '''
    This fixture should be implemented to overwrite default salt master
    configuration options.

    It will be applied over the loaded default options
    '''


@pytest.fixture
def master_of_masters_config_overrides():
    '''
    This fixture should be implemented to overwrite default salt master
    configuration options.

    It will be applied over the loaded default options
    '''


@pytest.fixture
def minion_config_overrides():
    '''
    This fixture should be implemented to overwrite default salt minion
    configuration options.

    It will be applied over the loaded default options
    '''


@pytest.fixture
def secondary_minion_config_overrides():
    '''
    This fixture should be implemented to overwrite default secondary salt minion
    configuration options.

    It will be applied over the loaded default options
    '''


@pytest.fixture
def syndic_config_overrides():
    '''
    This fixture should be implemented to overwrite default salt syndic
    minion configuration options.

    It will be applied over the loaded default options
    '''


@pytest.fixture
def proxy_config_overrides():
    '''
    This fixture should be implemented to overwrite default salt proxy
    minion configuration options.

    It will be applied over the loaded default options
    '''


@pytest.fixture(scope='session')
def session_master_config_file(session_conf_dir):
    '''
    Returns the path to the salt master configuration file
    '''
    return session_conf_dir.join('master').realpath().strpath


@pytest.fixture(scope='session')
def session_master_of_masters_config_file(session_master_of_masters_conf_dir):
    '''
    Returns the path to the salt master configuration file
    '''
    return session_master_of_masters_conf_dir.join('master').realpath().strpath


@pytest.fixture(scope='session')
def session_minion_config_file(session_conf_dir):
    '''
    Returns the path to the salt minion configuration file
    '''
    return session_conf_dir.join('minion').realpath().strpath


@pytest.fixture(scope='session')
def session_secondary_minion_config_file(session_secondary_conf_dir):
    '''
    Returns the path to the salt secondary minion configuration file
    '''
    return session_secondary_conf_dir.join('minion').realpath().strpath


@pytest.fixture(scope='session')
def session_proxy_config_file(session_conf_dir):
    '''
    Returns the path to the salt proxy configuration file
    '''
    return session_conf_dir.join('proxy').realpath().strpath


@pytest.fixture(scope='session')
def session_master_config_overrides():
    '''
    This fixture should be implemented to overwrite default salt master
    configuration options.

    It will be applied over the loaded default options
    '''


@pytest.fixture(scope='session')
def session_master_of_masters_config_overrides():
    '''
    This fixture should be implemented to overwrite default salt master
    configuration options.

    It will be applied over the loaded default options
    '''


@pytest.fixture(scope='session')
def session_minion_config_overrides():
    '''
    This fixture should be implemented to overwrite default salt minion
    configuration options.

    It will be applied over the loaded default options
    '''


@pytest.fixture(scope='session')
def session_secondary_minion_config_overrides():
    '''
    This fixture should be implemented to overwrite default secondary salt minion
    configuration options.

    It will be applied over the loaded default options
    '''


@pytest.fixture(scope='session')
def session_syndic_config_overrides():
    '''
    This fixture should be implemented to overwrite default salt syndic
    minion configuration options.

    It will be applied over the loaded default options
    '''


@pytest.fixture(scope='session')
def session_proxy_config_overrides():
    '''
    This fixture should be implemented to overwrite default salt proxy
    minion configuration options.

    It will be applied over the loaded default options
    '''


@pytest.fixture
def master_log_prefix(master_id):
    return 'salt-master/{}'.format(master_id)


@pytest.fixture(scope='session')
def session_master_log_prefix(session_master_id):
    return 'salt-master/{}'.format(session_master_id)


@pytest.fixture
def master_of_masters_log_prefix(master_of_masters_id):
    return 'salt-master/{}'.format(master_of_masters_id)


@pytest.fixture(scope='session')
def session_master_of_masters_log_prefix(session_master_of_masters_id):
    return 'salt-master/{}'.format(session_master_of_masters_id)


@pytest.fixture
def minion_log_prefix(minion_id):
    return 'salt-minion/{}'.format(minion_id)


@pytest.fixture(scope='session')
def session_minion_log_prefix(session_minion_id):
    return 'salt-minion/{}'.format(session_minion_id)


@pytest.fixture
def proxy_log_prefix(minion_id):
    return 'salt-proxy/{}'.format(minion_id)


@pytest.fixture(scope='session')
def session_proxy_log_prefix(session_minion_id):
    return 'salt-proxy/{}'.format(session_minion_id)


@pytest.fixture
def secondary_minion_log_prefix(secondary_minion_id):
    return 'salt-minion/{}'.format(secondary_minion_id)


@pytest.fixture(scope='session')
def session_secondary_minion_log_prefix(session_secondary_minion_id):
    return 'salt-minion/{}'.format(session_secondary_minion_id)


@pytest.fixture
def syndic_log_prefix(syndic_id):
    return 'salt-syndic/{}'.format(syndic_id)


@pytest.fixture(scope='session')
def session_syndic_log_prefix(session_syndic_id):
    return 'salt-syndic/{}'.format(session_syndic_id)


@pytest.fixture
def salt_log_prefix(minion_id):
    return 'salt/{}'.format(minion_id)


@pytest.fixture(scope='session')
def session_salt_log_prefix(session_minion_id):
    return 'salt/{}'.format(session_minion_id)


@pytest.fixture
def salt_call_log_prefix(master_id):
    return 'salt-call/{}'.format(master_id)


@pytest.fixture(scope='session')
def session_salt_call_log_prefix(session_master_id):
    return 'salt-call/{}'.format(session_master_id)


@pytest.fixture
def salt_key_log_prefix(master_id):
    return 'salt-key/{}'.format(master_id)


@pytest.fixture(scope='session')
def session_salt_key_log_prefix(session_master_id):
    return 'salt-key/{}'.format(session_master_id)


@pytest.fixture
def salt_run_log_prefix(master_id):
    return 'salt-run/{}'.format(master_id)


@pytest.fixture(scope='session')
def session_salt_run_log_prefix(session_master_id):
    return 'salt-run/{}'.format(session_master_id)


def apply_master_config(default_options,
                        root_dir,
                        config_file,
                        publish_port,
                        return_port,
                        engine_port,
                        config_overrides,
                        master_id,
                        base_env_state_tree_root_dirs,
                        prod_env_state_tree_root_dirs,
                        base_env_pillar_tree_root_dirs,
                        prod_env_pillar_tree_root_dirs,
                        running_username,
                        log_server_port,
                        log_server_level,
                        engines_dir,
                        log_handlers_dir,
                        master_log_prefix,
                        tcp_master_pub_port,
                        tcp_master_pull_port,
                        tcp_master_publish_pull,
                        tcp_master_workers,
                        direct_overrides=None):
    '''
    This fixture will return the salt master configuration options after being
    overridden with any options passed from ``master_config_overrides``
    '''
    import pytestsalt.utils.compat as compat
    import salt.config
    import salt.utils
    import salt.utils.dictupdate as dictupdate
    import salt.utils.verify as salt_verify
    import salt.utils.yaml as yamlserialize
    _default_options = {
        'id': master_id,
        'interface': '127.0.0.1',
        'root_dir': root_dir.strpath,
        'publish_port': publish_port,
        'ret_port': return_port,
        'tcp_master_pub_port': tcp_master_pub_port,
        'tcp_master_pull_port': tcp_master_pull_port,
        'tcp_master_publish_pull': tcp_master_publish_pull,
        'tcp_master_workers': tcp_master_workers,
        'worker_threads': 3,
        'pidfile': 'run/master.pid',
        'pki_dir': 'pki',
        'cachedir': 'cache',
        'timeout': 3,
        'sock_dir': '.salt-unix',
        'open_mode': True,
        'syndic_master': 'localhost',
        'fileserver_list_cache_time': 0,
        'fileserver_backend': ['roots'],
        'pillar_opts': False,
        'peer': {
            '.*': [
                'test.*'
            ]
        },
        'log_file': 'logs/master.log',
        'log_level_logfile': 'debug',
        'key_logfile': 'logs/key.log',
        'token_dir': 'tokens',
        'token_file': root_dir.join('ksfjhdgiuebfgnkefvsikhfjdgvkjahcsidk').strpath,
        'file_buffer_size': 8192,
        'user': running_username,
        'log_fmt_console': "[%(levelname)-8s][%(name)-5s:%(lineno)-4d] %(message)s",
        'log_fmt_logfile': "[%(asctime)s,%(msecs)03.0f][%(name)-5s:%(lineno)-4d][%(levelname)-8s] %(message)s",
        'file_roots': {
            'base': base_env_state_tree_root_dirs,
            'prod': prod_env_state_tree_root_dirs,
        },
        'pillar_roots': {
            'base': base_env_pillar_tree_root_dirs,
            'prod': prod_env_pillar_tree_root_dirs,
        },
        'hash_type': 'sha256'
    }
    for varname in ('sock_dir',):
        # These are settings which are tested against and provided by Salt's test suite, so,
        # let's not override them if provided
        if varname in default_options:
            _default_options.pop(varname)
    # Merge in the initial default options with the internal _default_options
    dictupdate.update(default_options, _default_options, merge_lists=True)

    if config_overrides:
        # Merge in the default options with the master_config_overrides
        dictupdate.update(default_options, config_overrides, merge_lists=True)

    default_options.setdefault('engines', [])
    if 'pytest' not in default_options['engines']:
        default_options['engines'].append('pytest')

    if 'engines_dirs' not in default_options:
        default_options['engines_dirs'] = []

    default_options['engines_dirs'].insert(0, engines_dir)
    default_options['pytest_engine_port'] = engine_port

    if 'log_handlers_dirs' not in default_options:
        default_options['log_handlers_dirs'] = []
    default_options['log_handlers_dirs'].insert(0, log_handlers_dir)

    default_options['pytest_log_host'] = 'localhost'
    default_options['pytest_log_port'] = log_server_port
    default_options['pytest_log_level'] = log_server_level
    default_options['pytest_log_prefix'] = master_log_prefix

    if direct_overrides is not None:
        # We've been passed some direct override configuration.
        # Apply it!
        dictupdate.update(default_options, direct_overrides, merge_lists=True)

    log.debug('Writing to configuration file %s. Configuration:\n%s',
              config_file,
              pprint.pformat(default_options))

    # Write down the computed configuration into the config file
    with compat.fopen(config_file, 'w') as wfh:
        yamlserialize.safe_dump(default_options, wfh, default_flow_style=False)

    # Make sure to load the config file as a salt-master starting from CLI
    options = salt.config.master_config(config_file)

    # verify env to make sure all required directories are created and have the
    # right permissions
    verify_env_entries = [
        os.path.join(options['pki_dir'], 'minions'),
        os.path.join(options['pki_dir'], 'minions_pre'),
        os.path.join(options['pki_dir'], 'minions_rejected'),
        os.path.join(options['pki_dir'], 'accepted'),
        os.path.join(options['pki_dir'], 'rejected'),
        os.path.join(options['pki_dir'], 'pending'),
        os.path.dirname(options['log_file']),
        options['token_dir'],
        #options['extension_modules'],
        options['sock_dir'],
    ]
    verify_env_entries += base_env_state_tree_root_dirs
    verify_env_entries += prod_env_state_tree_root_dirs
    verify_env_entries += base_env_pillar_tree_root_dirs
    verify_env_entries += prod_env_pillar_tree_root_dirs

    if default_options.get('transport', None) == 'raet':
        verify_env_entries.extend([
            os.path.join(options['cachedir'], 'raet'),
        ])
    else:
        verify_env_entries.extend([
            os.path.join(options['cachedir'], 'jobs'),
        ])

    try:
        salt_verify.verify_env(  # pylint: disable=unexpected-keyword-arg
            verify_env_entries,
            running_username,
            sensitive_dirs=[options['pki_dir']]
        )
    except TypeError:
        salt_verify.verify_env(
            verify_env_entries,
            running_username,
            pki_dir=options['pki_dir']
        )
    return options


@pytest.fixture
def master_default_options():
    return {}


@pytest.fixture
def master_config(root_dir,
                  master_default_options,
                  master_config_file,
                  master_publish_port,
                  master_return_port,
                  master_engine_port,
                  master_config_overrides,
                  master_id,
                  base_env_state_tree_root_dir,
                  prod_env_state_tree_root_dir,
                  base_env_pillar_tree_root_dir,
                  prod_env_pillar_tree_root_dir,
                  running_username,
                  log_server_port,
                  log_server_level,
                  engines_dir,
                  log_handlers_dir,
                  master_log_prefix,
                  master_tcp_master_pub_port,
                  master_tcp_master_pull_port,
                  master_tcp_master_publish_pull,
                  master_tcp_master_workers):
    '''
    This fixture will return the salt master configuration options after being
    overridden with any options passed from ``master_config_overrides``
    '''
    return apply_master_config(master_default_options,
                               root_dir,
                               master_config_file,
                               master_publish_port,
                               master_return_port,
                               master_engine_port,
                               master_config_overrides,
                               master_id,
                               [base_env_state_tree_root_dir.strpath],
                               [prod_env_state_tree_root_dir.strpath],
                               [base_env_pillar_tree_root_dir.strpath],
                               [prod_env_pillar_tree_root_dir.strpath],
                               running_username,
                               log_server_port,
                               log_server_level,
                               engines_dir,
                               log_handlers_dir,
                               master_log_prefix,
                               master_tcp_master_pub_port,
                               master_tcp_master_pull_port,
                               master_tcp_master_publish_pull,
                               master_tcp_master_workers)


@pytest.fixture(scope='session')
def session_master_default_options():
    return {}


@pytest.fixture(scope='session')
def session_master_config(session_root_dir,
                          session_master_default_options,
                          session_master_config_file,
                          session_master_publish_port,
                          session_master_return_port,
                          session_master_engine_port,
                          session_master_config_overrides,
                          session_master_id,
                          session_base_env_state_tree_root_dir,
                          session_prod_env_state_tree_root_dir,
                          session_base_env_pillar_tree_root_dir,
                          session_prod_env_pillar_tree_root_dir,
                          running_username,
                          log_server_port,
                          log_server_level,
                          engines_dir,
                          log_handlers_dir,
                          session_master_log_prefix,
                          session_master_tcp_master_pub_port,
                          session_master_tcp_master_pull_port,
                          session_master_tcp_master_publish_pull,
                          session_master_tcp_master_workers):
    '''
    This fixture will return the salt master configuration options after being
    overridden with any options passed from ``session_master_config_overrides``
    '''
    return apply_master_config(session_master_default_options,
                               session_root_dir,
                               session_master_config_file,
                               session_master_publish_port,
                               session_master_return_port,
                               session_master_engine_port,
                               session_master_config_overrides,
                               session_master_id,
                               [session_base_env_state_tree_root_dir.strpath],
                               [session_prod_env_state_tree_root_dir.strpath],
                               [session_base_env_pillar_tree_root_dir.strpath],
                               [session_prod_env_pillar_tree_root_dir.strpath],
                               running_username,
                               log_server_port,
                               log_server_level,
                               engines_dir,
                               log_handlers_dir,
                               session_master_log_prefix,
                               session_master_tcp_master_pub_port,
                               session_master_tcp_master_pull_port,
                               session_master_tcp_master_publish_pull,
                               session_master_tcp_master_workers)


@pytest.fixture(scope='session')
def master_of_masters_default_options():
    return {}


@pytest.fixture
def master_of_masters_config(master_of_masters_root_dir,
                             master_of_masters_config_file,
                             master_of_masters_default_options,
                             master_of_masters_publish_port,
                             master_of_masters_return_port,
                             master_of_masters_engine_port,
                             master_of_masters_config_overrides,
                             master_of_masters_id,
                             master_of_masters_base_env_state_tree_root_dir,
                             master_of_masters_prod_env_state_tree_root_dir,
                             master_of_masters_base_env_pillar_tree_root_dir,
                             master_of_masters_prod_env_pillar_tree_root_dir,
                             running_username,
                             log_server_port,
                             log_server_level,
                             engines_dir,
                             log_handlers_dir,
                             master_of_masters_log_prefix,
                             master_of_masters_master_tcp_master_pub_port,
                             master_of_masters_master_tcp_master_pull_port,
                             master_of_masters_master_tcp_master_publish_pull,
                             master_of_masters_master_tcp_master_workers):
    '''
    This fixture will return the salt master configuration options after being
    overridden with any options passed from ``master_config_overrides``
    '''
    direct_overrides = {
        'order_masters': True,
    }
    return apply_master_config(master_of_masters_default_options,
                               master_of_masters_root_dir,
                               master_of_masters_config_file,
                               master_of_masters_publish_port,
                               master_of_masters_return_port,
                               master_of_masters_engine_port,
                               master_of_masters_config_overrides,
                               master_of_masters_id,
                               [master_of_masters_base_env_state_tree_root_dir.strpath],
                               [master_of_masters_prod_env_state_tree_root_dir.strpath],
                               [master_of_masters_base_env_pillar_tree_root_dir.strpath],
                               [master_of_masters_prod_env_pillar_tree_root_dir.strpath],
                               running_username,
                               log_server_port,
                               log_server_level,
                               engines_dir,
                               log_handlers_dir,
                               master_of_masters_log_prefix,
                               master_of_masters_master_tcp_master_pub_port,
                               master_of_masters_master_tcp_master_pull_port,
                               master_of_masters_master_tcp_master_publish_pull,
                               master_of_masters_master_tcp_master_workers,
                               direct_overrides=direct_overrides)


@pytest.fixture(scope='session')
def session_master_of_masters_default_options():
    return {}


@pytest.fixture(scope='session')
def session_master_of_masters_config(session_master_of_masters_root_dir,
                                     session_master_of_masters_config_file,
                                     session_master_of_masters_publish_port,
                                     session_master_of_masters_return_port,
                                     session_master_of_masters_engine_port,
                                     session_master_of_masters_default_options,
                                     session_master_of_masters_config_overrides,
                                     session_master_of_masters_id,
                                     session_base_env_state_tree_root_dir,
                                     session_prod_env_state_tree_root_dir,
                                     session_base_env_pillar_tree_root_dir,
                                     session_prod_env_pillar_tree_root_dir,
                                     running_username,
                                     log_server_port,
                                     log_server_level,
                                     engines_dir,
                                     log_handlers_dir,
                                     session_master_of_masters_log_prefix,
                                     session_master_of_masters_master_tcp_master_pub_port,
                                     session_master_of_masters_master_tcp_master_pull_port,
                                     session_master_of_masters_master_tcp_master_publish_pull,
                                     session_master_of_masters_master_tcp_master_workers):
    '''
    This fixture will return the salt master configuration options after being
    overridden with any options passed from ``session_master_config_overrides``
    '''
    direct_overrides = {
        'order_masters': True,
    }
    return apply_master_config(session_master_of_masters_default_options,
                               session_master_of_masters_root_dir,
                               session_master_of_masters_config_file,
                               session_master_of_masters_publish_port,
                               session_master_of_masters_return_port,
                               session_master_of_masters_engine_port,
                               session_master_of_masters_config_overrides,
                               session_master_of_masters_id,
                               [session_base_env_state_tree_root_dir.strpath],
                               [session_prod_env_state_tree_root_dir.strpath],
                               [session_base_env_pillar_tree_root_dir.strpath],
                               [session_prod_env_pillar_tree_root_dir.strpath],
                               running_username,
                               log_server_port,
                               log_server_level,
                               engines_dir,
                               log_handlers_dir,
                               session_master_of_masters_log_prefix,
                               session_master_of_masters_master_tcp_master_pub_port,
                               session_master_of_masters_master_tcp_master_pull_port,
                               session_master_of_masters_master_tcp_master_publish_pull,
                               session_master_of_masters_master_tcp_master_workers,
                               direct_overrides=direct_overrides)


def apply_minion_config(default_options,
                        root_dir,
                        config_file,
                        return_port,
                        config_overrides,
                        minion_id,
                        running_username,
                        log_server_port,
                        log_server_level,
                        log_handlers_dir,
                        minion_log_prefix,
                        tcp_pub_port,
                        tcp_pull_port,
                        direct_overrides=None):
    '''
    This fixture will return the salt minion configuration options after being
    overridden with any options passed from ``config_overrides``
    '''
    import pytestsalt.utils.compat as compat
    import salt.config
    import salt.utils
    import salt.utils.dictupdate as dictupdate
    import salt.utils.verify as salt_verify
    import salt.utils.yaml as yamlserialize
    _default_options = {
        'root_dir': root_dir.strpath,
        'interface': '127.0.0.1',
        'master': '127.0.0.1',
        'master_port': return_port,
        'tcp_pub_port': tcp_pub_port,
        'tcp_pull_port': tcp_pull_port,
        'id': minion_id,
        'pidfile': 'run/minion.pid',
        'pki_dir': 'pki',
        'cachedir': 'cache',
        'sock_dir': '.salt-unix',
        'log_file': 'logs/minion.log',
        'log_level_logfile': 'debug',
        'loop_interval': 0.05,
        'open_mode': True,
        'user': running_username,
        #'multiprocessing': False,
        'log_fmt_console': "[%(levelname)-8s][%(name)-5s:%(lineno)-4d] %(message)s",
        'log_fmt_logfile': "[%(asctime)s,%(msecs)03.0f][%(name)-5s:%(lineno)-4d][%(levelname)-8s] %(message)s",
        'hash_type': 'sha256'
    }
    for varname in ('sock_dir',):
        # These are settings which are tested against and provided by Salt's test suite, so,
        # let's not override them if provided
        if varname in default_options:
            _default_options.pop(varname)
    # Merge in the initial default options with the internal _default_options
    dictupdate.update(default_options, _default_options, merge_lists=True)

    if config_overrides:
        # Merge in the default options with the minion_config_overrides
        dictupdate.update(default_options, config_overrides, merge_lists=True)

    if 'log_handlers_dirs' not in default_options:
        default_options['log_handlers_dirs'] = []
    default_options['log_handlers_dirs'].insert(0, log_handlers_dir)

    default_options['pytest_log_host'] = 'localhost'
    default_options['pytest_log_port'] = log_server_port
    default_options['pytest_log_level'] = log_server_level
    default_options['pytest_log_prefix'] = minion_log_prefix

    if direct_overrides is not None:
        # We've been passed some direct override configuration.
        # Apply it!
        dictupdate.update(default_options, direct_overrides, merge_lists=True)

    log.debug('Writing to configuration file %s. Configuration:\n%s',
              config_file,
              pprint.pformat(default_options))

    # Write down the computed configuration into the config file
    with compat.fopen(config_file, 'w') as wfh:
        yamlserialize.safe_dump(default_options, wfh, default_flow_style=False)

    # Make sure to load the config file as a salt-master starting from CLI
    options = salt.config.minion_config(config_file)

    # verify env to make sure all required directories are created and have the
    # right permissions
    verify_env_entries = [
        os.path.join(options['pki_dir'], 'minions'),
        os.path.join(options['pki_dir'], 'minions_pre'),
        os.path.join(options['pki_dir'], 'minions_rejected'),
        os.path.join(options['pki_dir'], 'accepted'),
        os.path.join(options['pki_dir'], 'rejected'),
        os.path.join(options['pki_dir'], 'pending'),
        os.path.dirname(options['log_file']),
        os.path.join(options['cachedir'], 'proc'),
        #options['extension_modules'],
        options['sock_dir'],
    ]
    try:
        # Salt > v2017.7.x
        salt_verify.verify_env(  # pylint: disable=unexpected-keyword-arg
            verify_env_entries,
            running_username,
            sensitive_dirs=[options['pki_dir']]
        )
    except TypeError:
        # Salt <= v2017.7.x
        salt_verify.verify_env(
            verify_env_entries,
            running_username,
            pki_dir=options['pki_dir']
        )
    return options


def apply_proxy_config(default_options,
                       root_dir,
                       config_file,
                       return_port,
                       config_overrides,
                       proxy_id,
                       running_username,
                       log_server_port,
                       log_server_level,
                       log_handlers_dir,
                       proxy_log_prefix,
                       tcp_pub_port,
                       tcp_pull_port,
                       direct_overrides=None):
    '''
    This fixture will return the salt proxy configuration options after being
    overridden with any options passed from ``config_overrides``
    '''
    import pytestsalt.utils.compat as compat
    import salt.config
    import salt.utils
    import salt.utils.dictupdate as dictupdate
    import salt.utils.verify as salt_verify
    import salt.utils.yaml as yamlserialize
    _default_options = {
        'root_dir': root_dir.strpath,
        'interface': '127.0.0.1',
        'master': '127.0.0.1',
        'master_port': return_port,
        'tcp_pub_port': tcp_pub_port,
        'tcp_pull_port': tcp_pull_port,
        'id': proxy_id,
        'pidfile': 'run/proxy.pid',
        'pki_dir': 'pki',
        'cachedir': 'cache',
        'sock_dir': '.salt-unix',
        'log_file': 'logs/proxy.log',
        'log_level_logfile': 'debug',
        'loop_interval': 0.05,
        'open_mode': True,
        'user': running_username,
        'log_fmt_console': "[%(levelname)-8s][%(name)-5s:%(lineno)-4d] %(message)s",
        'log_fmt_logfile': "[%(asctime)s,%(msecs)03.0f][%(name)-5s:%(lineno)-4d][%(levelname)-8s] %(message)s",
        'hash_type': 'sha256',
        'add_proxymodule_to_opts': False,
        'proxy': {'proxytype': 'dummy'}
    }
    for varname in ('sock_dir',):
        # These are settings which are tested against and provided by Salt's test suite, so,
        # let's not override them if provided
        if varname in default_options:
            _default_options.pop(varname)
    # Merge in the initial default options with the internal _default_options
    dictupdate.update(default_options, _default_options, merge_lists=True)

    if config_overrides:
        # Merge in the default options with the proxy_config_overrides
        dictupdate.update(default_options, config_overrides, merge_lists=True)

    if 'log_handlers_dirs' not in default_options:
        default_options['log_handlers_dirs'] = []
    default_options['log_handlers_dirs'].insert(0, log_handlers_dir)

    default_options['pytest_log_host'] = 'localhost'
    default_options['pytest_log_port'] = log_server_port
    default_options['pytest_log_level'] = log_server_level
    default_options['pytest_log_prefix'] = proxy_log_prefix

    if direct_overrides is not None:
        # We've been passed some direct override configuration.
        # Apply it!
        dictupdate.update(default_options, direct_overrides, merge_lists=True)

    log.debug('Writing to configuration file %s. Configuration:\n%s',
              config_file,
              pprint.pformat(default_options))

    # Write down the computed configuration into the config file
    with compat.fopen(config_file, 'w') as wfh:
        yamlserialize.safe_dump(default_options, wfh, default_flow_style=False)

    # Make sure to load the config file as a salt-master starting from CLI
    options = salt.config.proxy_config(config_file)

    # verify env to make sure all required directories are created and have the
    # right permissions
    verify_env_entries = [
        os.path.dirname(options['log_file']),
        options['sock_dir'],
    ]
    try:
        # Salt > v2017.7.x
        salt_verify.verify_env(  # pylint: disable=unexpected-keyword-arg
            verify_env_entries,
            running_username,
            sensitive_dirs=[options['pki_dir']]
        )
    except TypeError:
        # Salt <= v2017.7.x
        salt_verify.verify_env(
            verify_env_entries,
            running_username,
            pki_dir=options['pki_dir']
        )
    return options


@pytest.fixture
def minion_default_options():
    return {}


@pytest.fixture
def minion_config(root_dir,
                  minion_config_file,
                  master_return_port,
                  minion_default_options,
                  minion_config_overrides,
                  minion_id,
                  running_username,
                  log_server_port,
                  log_server_level,
                  log_handlers_dir,
                  minion_log_prefix,
                  minion_tcp_pub_port,
                  minion_tcp_pull_port):
    '''
    This fixture will return the salt minion configuration options after being
    overrided with any options passed from ``minion_config_overrides``
    '''
    return apply_minion_config(minion_default_options,
                               root_dir,
                               minion_config_file,
                               master_return_port,
                               minion_config_overrides,
                               minion_id,
                               running_username,
                               log_server_port,
                               log_server_level,
                               log_handlers_dir,
                               minion_log_prefix,
                               minion_tcp_pub_port,
                               minion_tcp_pull_port)


@pytest.fixture(scope='session')
def session_minion_default_options():
    return {}


@pytest.fixture(scope='session')
def session_minion_config(session_root_dir,
                          session_minion_config_file,
                          session_master_return_port,
                          session_minion_default_options,
                          session_minion_config_overrides,
                          session_minion_id,
                          running_username,
                          log_server_port,
                          log_server_level,
                          log_handlers_dir,
                          session_minion_log_prefix,
                          session_minion_tcp_pub_port,
                          session_minion_tcp_pull_port):
    '''
    This fixture will return the session salt minion configuration options after being
    overrided with any options passed from ``session_minion_config_overrides``
    '''
    return apply_minion_config(session_minion_default_options,
                               session_root_dir,
                               session_minion_config_file,
                               session_master_return_port,
                               session_minion_config_overrides,
                               session_minion_id,
                               running_username,
                               log_server_port,
                               log_server_level,
                               log_handlers_dir,
                               session_minion_log_prefix,
                               session_minion_tcp_pub_port,
                               session_minion_tcp_pull_port)


@pytest.fixture
def secondary_minion_default_options():
    return {}


@pytest.fixture
def secondary_minion_config(secondary_root_dir,
                            secondary_minion_config_file,
                            master_return_port,
                            secondary_minion_default_options,
                            secondary_minion_config_overrides,
                            secondary_minion_id,
                            running_username,
                            log_server_port,
                            log_server_level,
                            log_handlers_dir,
                            secondary_minion_log_prefix,
                            secondary_minion_tcp_pub_port,
                            secondary_minion_tcp_pull_port):
    '''
    This fixture will return the secondary salt minion configuration options after being
    overrided with any options passed from ``secondary_minion_config_overrides``
    '''
    return apply_minion_config(secondary_minion_default_options,
                               secondary_root_dir,
                               secondary_minion_config_file,
                               master_return_port,
                               secondary_minion_config_overrides,
                               secondary_minion_id,
                               running_username,
                               log_server_port,
                               log_server_level,
                               log_handlers_dir,
                               secondary_minion_log_prefix,
                               secondary_minion_tcp_pub_port,
                               secondary_minion_tcp_pull_port)


@pytest.fixture(scope='session')
def session_secondary_minion_default_options():
    return {}


@pytest.fixture(scope='session')
def session_secondary_minion_config(session_secondary_root_dir,
                                    session_secondary_minion_config_file,
                                    session_master_return_port,
                                    session_secondary_minion_default_options,
                                    session_secondary_minion_config_overrides,
                                    session_secondary_minion_id,
                                    running_username,
                                    log_server_port,
                                    log_server_level,
                                    log_handlers_dir,
                                    session_secondary_minion_log_prefix,
                                    session_secondary_minion_tcp_pub_port,
                                    session_secondary_minion_tcp_pull_port):
    '''
    This fixture will return the session salt minion configuration options after being
    overrided with any options passed from ``session_secondary_minion_config_overrides``
    '''
    return apply_minion_config(session_secondary_minion_default_options,
                               session_secondary_root_dir,
                               session_secondary_minion_config_file,
                               session_master_return_port,
                               session_secondary_minion_config_overrides,
                               session_secondary_minion_id,
                               running_username,
                               log_server_port,
                               log_server_level,
                               log_handlers_dir,
                               session_secondary_minion_log_prefix,
                               session_secondary_minion_tcp_pub_port,
                               session_secondary_minion_tcp_pull_port)


def apply_syndic_config(syndic_default_options,
                        master_config_file,
                        syndic_conf_dir,
                        syndic_config_overrides,
                        running_username,
                        log_server_port,
                        log_server_level,
                        syndic_log_prefix,
                        syndic_id,
                        log_handlers_dir,
                        root_dir):
    '''
    This fixture will return the salt syndic configuration options after being
    overridden with any options passed from ``config_overrides``
    '''
    import pytestsalt.utils.compat as compat
    import salt.config
    import salt.utils
    import salt.utils.dictupdate as dictupdate
    import salt.utils.yaml as yamlserialize

    with compat.fopen(master_config_file) as rfh:
        master_config = yamlserialize.safe_load(rfh.read())

    master_overrides = {
        'syndic_master': 'localhost',
        'syndic_master_port': master_config['ret_port'],
        'syndic_pidfile': 'run/salt-syndic.pid',
        'syndic_user': running_username,
        'syndic_log_file': 'logs/syndic.log',
    }
    if syndic_default_options:
        # Override pytest salt overrides with any value comming in from syndic_default_options
        master_overrides.update(syndic_default_options)

    dictupdate.update(master_config, master_overrides, merge_lists=True)

    syndic_master_config_file = syndic_conf_dir.join('master').realpath().strpath

    # Write down the master computed configuration into the config file
    log.debug('Writing to configuration file %s. Configuration:\n%s',
              syndic_master_config_file,
              pprint.pformat(master_config))
    with compat.fopen(syndic_master_config_file, 'w') as wfh:
        yamlserialize.safe_dump(master_config, wfh, default_flow_style=False)

    syndic_config_file = syndic_conf_dir.join('minion').realpath().strpath
    direct_overrides = copy.deepcopy(master_overrides)
    direct_overrides.update({
        'id': syndic_id,
    })

    apply_minion_config(syndic_default_options,
                        root_dir,
                        syndic_config_file,
                        master_config['ret_port'],  # master_return_port
                        syndic_config_overrides,
                        syndic_id,
                        running_username,
                        log_server_port,
                        log_server_level,
                        log_handlers_dir,
                        syndic_log_prefix,
                        None,  # minion_tcp_pub_port,
                        None,  # minion_tcp_pull_port,
                        direct_overrides=direct_overrides)

    return salt.config.syndic_config(syndic_master_config_file, syndic_config_file)


@pytest.fixture
def syndic_default_options():
    return {}


@pytest.fixture
def syndic_config(master_config,
                  master_config_file,
                  syndic_conf_dir,
                  syndic_default_options,
                  syndic_config_overrides,
                  running_username,
                  log_server_port,
                  log_server_level,
                  syndic_log_prefix,
                  syndic_id,
                  log_handlers_dir,
                  root_dir):
    '''
    This fixture will return the salt syndic configuration options after being
    overridden with any options passed from ``syndic_config_overrides``
    '''
    return apply_syndic_config(syndic_default_options,
                               master_config_file,
                               syndic_conf_dir,
                               syndic_config_overrides,
                               running_username,
                               log_server_port,
                               log_server_level,
                               syndic_log_prefix,
                               syndic_id,
                               log_handlers_dir,
                               root_dir)


@pytest.fixture(scope='session')
def session_syndic_default_options():
    return {}


@pytest.fixture(scope='session')
def session_syndic_config(session_master_config_file,
                          session_syndic_conf_dir,
                          session_syndic_default_options,
                          session_syndic_config_overrides,
                          running_username,
                          log_server_port,
                          log_server_level,
                          session_syndic_log_prefix,
                          session_syndic_id,
                          log_handlers_dir,
                          session_root_dir):
    '''
    This fixture will return the salt syndic configuration options after being
    overridden with any options passed from ``syndic_config_overrides``
    '''
    return apply_syndic_config(session_syndic_default_options,
                               session_master_config_file,
                               session_syndic_conf_dir,
                               session_syndic_config_overrides,
                               running_username,
                               log_server_port,
                               log_server_level,
                               session_syndic_log_prefix,
                               session_syndic_id,
                               log_handlers_dir,
                               session_root_dir)


@pytest.fixture
def proxy_default_options():
    return {}


@pytest.fixture
def proxy_config(root_dir,
                 proxy_config_file,
                 master_return_port,
                 proxy_default_options,
                 proxy_config_overrides,
                 proxy_id,
                 running_username,
                 log_server_port,
                 log_server_level,
                 log_handlers_dir,
                 proxy_log_prefix,
                 proxy_tcp_pub_port,
                 proxy_tcp_pull_port):
    '''
    This fixture will return the salt proxy configuration options after being
    overrided with any options passed from ``proxy_config_overrides``
    '''
    return apply_proxy_config(proxy_default_options,
                              root_dir,
                              proxy_config_file,
                              master_return_port,
                              proxy_config_overrides,
                              proxy_id,
                              running_username,
                              log_server_port,
                              log_server_level,
                              log_handlers_dir,
                              proxy_log_prefix,
                              proxy_tcp_pub_port,
                              proxy_tcp_pull_port)


@pytest.fixture(scope='session')
def session_proxy_default_options():
    return {}


@pytest.fixture(scope='session')
def session_proxy_config(session_root_dir,
                         session_proxy_config_file,
                         session_master_return_port,
                         session_proxy_default_options,
                         session_proxy_config_overrides,
                         session_proxy_id,
                         running_username,
                         log_server_port,
                         log_server_level,
                         log_handlers_dir,
                         session_proxy_log_prefix,
                         session_proxy_tcp_pub_port,
                         session_proxy_tcp_pull_port):
    '''
    This fixture will return the session salt proxy configuration options after being
    overrided with any options passed from ``session_proxy_config_overrides``
    '''
    return apply_proxy_config(session_proxy_default_options,
                              session_root_dir,
                              session_proxy_config_file,
                              session_master_return_port,
                              session_proxy_config_overrides,
                              session_proxy_id,
                              running_username,
                              log_server_port,
                              log_server_level,
                              log_handlers_dir,
                              session_proxy_log_prefix,
                              session_proxy_tcp_pub_port,
                              session_proxy_tcp_pull_port)


@pytest.fixture
def salt_ssh_log_prefix(sshd_port):
    return 'salt-ssh/{}'.format(sshd_port)


@pytest.fixture(scope='session')
def session_salt_ssh_log_prefix(session_sshd_port):
    return 'salt-ssh/{}'.format(session_sshd_port)


@pytest.fixture
def ssh_client_key(ssh_config_dir):
    '''
    The ssh client key
    '''
    return _generate_ssh_key(ssh_config_dir.join('test_key').realpath().strpath)


@pytest.fixture(scope='session')
def session_ssh_client_key(session_ssh_config_dir):
    '''
    The ssh client key
    '''
    return _generate_ssh_key(session_ssh_config_dir.join('test_key').realpath().strpath)


def _sshd_config_lines(sshd_port):
    '''
    Return a list of lines which will make the sshd_config file
    '''
    return [
        'Port {}'.format(sshd_port),
        'ListenAddress 127.0.0.1',
        'Protocol 2',
        'UsePrivilegeSeparation yes',
        '# Turn strict modes off so that we can operate in /tmp',
        'StrictModes no',
        '# Lifetime and size of ephemeral version 1 server key',
        'KeyRegenerationInterval 3600',
        'ServerKeyBits 1024',
        '# Logging',
        'SyslogFacility AUTH',
        'LogLevel INFO',
        '# Authentication:',
        'LoginGraceTime 120',
        'PermitRootLogin without-password',
        'StrictModes yes',
        'RSAAuthentication yes',
        'PubkeyAuthentication yes',
        '#AuthorizedKeysFile	%h/.ssh/authorized_keys',
        '#AuthorizedKeysFile	key_test.pub',
        '# Don\'t read the user\'s ~/.rhosts and ~/.shosts files',
        'IgnoreRhosts yes',
        '# For this to work you will also need host keys in /etc/ssh_known_hosts',
        'RhostsRSAAuthentication no',
        '# similar for protocol version 2',
        'HostbasedAuthentication no',
        '#IgnoreUserKnownHosts yes',
        '# To enable empty passwords, change to yes (NOT RECOMMENDED)',
        'PermitEmptyPasswords no',
        '# Change to yes to enable challenge-response passwords (beware issues with',
        '# some PAM modules and threads)',
        'ChallengeResponseAuthentication no',
        '# Change to no to disable tunnelled clear text passwords',
        'PasswordAuthentication no',
        'X11Forwarding no',
        'X11DisplayOffset 10',
        'PrintMotd no',
        'PrintLastLog yes',
        'TCPKeepAlive yes',
        '#UseLogin no',
        'AcceptEnv LANG LC_*',
        'Subsystem sftp /usr/lib/openssh/sftp-server',
        '#UsePAM yes',
    ]


@pytest.fixture
def sshd_config_lines(sshd_port):
    '''
    Return a list of lines which will make the sshd_config file
    '''
    return _sshd_config_lines(sshd_port)


@pytest.fixture(scope='session')
def session_sshd_config_lines(session_sshd_port):
    '''
    Return a list of lines which will make the sshd_config file
    '''
    return _sshd_config_lines(session_sshd_port)


@pytest.fixture
def write_sshd_config(sshd_config_dir, sshd_config_lines, ssh_client_key):
    '''
    This fixture will write the necessary configuration to run an SSHD server to be used in tests
    '''
    return _write_sshd_config(sshd_config_dir, sshd_config_lines, ssh_client_key)


@pytest.fixture(scope='session')
def session_write_sshd_config(session_sshd_config_dir, session_sshd_config_lines, session_ssh_client_key):
    '''
    This fixture will write the necessary configuration to run an SSHD server to be used in tests
    '''
    return _write_sshd_config(session_sshd_config_dir, session_sshd_config_lines, session_ssh_client_key)


@pytest.fixture
def _write_sshd_config(sshd_config_dir, sshd_config_lines, ssh_client_key):
    '''
    This fixture will write the necessary configuration to run an SSHD server to be used in tests
    '''
    import pytestsalt.utils.compat as compat
    sshd = compat.which('sshd')

    if not sshd:
        pytest.skip('"sshd" not found.')

    # Generate server key
    server_key_dir = sshd_config_dir.realpath().strpath
    server_dsa_key_file = os.path.join(server_key_dir, 'ssh_host_dsa_key')
    server_ecdsa_key_file = os.path.join(server_key_dir, 'ssh_host_ecdsa_key')
    server_ed25519_key_file = os.path.join(server_key_dir, 'ssh_host_ed25519_key')

    _generate_ssh_key(server_dsa_key_file, 'dsa', 1024)
    _generate_ssh_key(server_ecdsa_key_file, 'ecdsa', 521)
    _generate_ssh_key(server_ed25519_key_file, 'ed25519', 521)

    sshd_config = sshd_config_lines[:]
    sshd_config.append('AuthorizedKeysFile {}.pub'.format(ssh_client_key))
    sshd_config.append('HostKey {}'.format(server_dsa_key_file))
    sshd_config.append('HostKey {}'.format(server_ecdsa_key_file))
    sshd_config.append('HostKey {}'.format(server_ed25519_key_file))

    with compat.fopen(sshd_config_dir.join('sshd_config').realpath().strpath, 'w') as wfh:
        wfh.write('\n'.join(sshd_config))


@pytest.fixture
def sshd_server_log_prefix(sshd_port):
    '''
    The log prefix to use for the sshd server fixture
    '''
    return 'sshd-server/{}'.format(sshd_port)


@pytest.fixture(scope='session')
def session_sshd_server_log_prefix(session_sshd_port):
    '''
    The log prefix to use for the sshd server fixture
    '''
    return 'session-sshd-server/{}'.format(session_sshd_port)


def _generate_ssh_key(key_path, key_type='ecdsa', key_size=521):
    '''
    Generate an SSH key
    '''
    import pytestsalt.utils.compat as compat
    log.debug('Generating ssh key(type: %s; size: %d; path: %s;)', key_type, key_size, key_path)
    keygen = compat.which('ssh-keygen')
    if not keygen:
        pytest.skip('"ssh-keygen" not found')

    keygen_process = subprocess.Popen(
        [
            keygen,
            '-t', key_type,
            '-b', str(key_size),
            '-C', '"$(whoami)@$(hostname)-$(date -I)"',
            '-f', os.path.basename(key_path),
            '-P', ''
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        close_fds=True,
        cwd=os.path.dirname(key_path)
    )
    _, keygen_err = keygen_process.communicate()
    if keygen_err:
        pytest.skip(
            'ssh-keygen had errors generating {}({}:{}): {}'.format(
                os.path.basename(key_path),
                key_type,
                key_size,
                keygen_err
            )
        )
    return key_path


@pytest.fixture
def roster_config_file(conf_dir):
    '''
    Returns the path to the salt-ssh roster configuration file
    '''
    return conf_dir.join('roster').realpath().strpath


@pytest.fixture(scope='session')
def session_roster_config_file(session_conf_dir):
    '''
    Returns the path to the salt-ssh roster configuration file
    '''
    return session_conf_dir.join('roster').realpath().strpath


@pytest.fixture
def roster_config_overrides():
    '''
    This fixture should be implemented to overwrite default salt-ssh roster
    configuration options.

    It will be applied over the loaded default options
    '''


@pytest.fixture(scope='session')
def session_roster_config_overrides():
    '''
    This fixture should be implemented to overwrite default salt-ssh roster
    configuration options.

    It will be applied over the loaded default options
    '''


def _roster_config(config_file, config, config_overrides):
    import pytestsalt.utils.compat as compat
    import salt.utils.yaml as yamlserialize
    if config_overrides:
        config.update(config_overrides)

    with compat.fopen(config_file, 'w') as wfh:
        yamlserialize.safe_dump(config, wfh, default_flow_style=False)
    return config


@pytest.fixture
def roster_config(roster_config_file,
                  roster_config_overrides,
                  running_username,
                  ssh_client_key,
                  master_config,
                  sshd_port):
    config = {
        'localhost': {
            'host': '127.0.0.1',
            'port': sshd_port,
            'user': running_username,
            'priv': ssh_client_key
        }
    }
    return _roster_config(roster_config_file, config, roster_config_overrides)


@pytest.fixture(scope='session')
def session_roster_config(session_roster_config_file,
                          session_roster_config_overrides,
                          running_username,
                          session_ssh_client_key,
                          session_sshd_port,
                          session_master_config):
    config = {
        'localhost': {
            'host': '127.0.0.1',
            'port': session_sshd_port,
            'user': running_username,
            'priv': session_ssh_client_key
        }
    }
    return _roster_config(session_roster_config_file, config, session_roster_config_overrides)


@pytest.mark.trylast
def pytest_configure(config):
    pytest.helpers.utils.register(apply_master_config)
    pytest.helpers.utils.register(apply_minion_config)
    pytest.helpers.utils.register(apply_syndic_config)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Pedro Algarvio (pedro@algarvio.me)`
    :copyright: Copyright 2015 by the SaltStack Team, see AUTHORS for more details.
    :license: Apache 2.0, see LICENSE for more details.


    test_salt_minion.py
    ~~~~~~~~~~~~~~~~~~~

    Test the pytest salt plugin salt minion
'''

# Import python libs
from __future__ import absolute_import


def test_salt_minion_running(salt_minion):
    assert salt_minion.is_alive()



# -*- coding: utf-8 -*-
'''
pytestsalt.salt.loader
~~~~~~~~~~~~~~~~~~~~~~

Salt loader support
'''

# Import python libs
import os

THIS_MODULE_DIR = os.path.abspath(os.path.dirname(__file__))


def engines_dirs():
    yield os.path.join(THIS_MODULE_DIR, 'engines')


def log_handlers_dirs():
    yield os.path.join(THIS_MODULE_DIR, 'log_handlers')



# -*- coding: utf-8 -*-
'''
    pytestsalt.utils.cli_scripts
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Code to generate Salt CLI scripts for test runs
'''

# Import Python Libs
from __future__ import absolute_import, unicode_literals
import os
import stat
import logging
import textwrap

log = logging.getLogger(__name__)

ROOT_DIR = os.path.dirname(os.path.dirname(__file__))

SCRIPT_TEMPLATES = {
    'salt': textwrap.dedent(
        '''
        from salt.scripts import salt_main

        if __name__ == '__main__':
            salt_main()
        '''
    ),
    'salt-api': textwrap.dedent(
        '''
        import salt.cli

        def main():
            sapi = salt.cli.SaltAPI()
            sapi.start()

        if __name__ == '__main__':
            main()'
        '''
    ),
    'common': textwrap.dedent(
        '''
        from salt.scripts import salt_{0}
        import salt.utils.platform

        def main():
            if salt.utils.platform.is_windows():
                import os.path
                import py_compile
                cfile = os.path.splitext(__file__)[0] + '.pyc'
                if not os.path.exists(cfile):
                    py_compile.compile(__file__, cfile)
            salt_{0}()

        if __name__ == '__main__':
            main()
        '''
    ),
    'coverage': textwrap.dedent(
        '''
        # Setup coverage environment variables
        COVERAGE_FILE = os.path.join(CODE_DIR, '.coverage')
        COVERAGE_PROCESS_START = os.path.join(CODE_DIR, '.coveragerc')
        os.environ[str('COVERAGE_FILE')] = str(COVERAGE_FILE)
        os.environ[str('COVERAGE_PROCESS_START')] = str(COVERAGE_PROCESS_START)
        '''
    ),
    'sitecustomize': textwrap.dedent(
        '''
        # Allow sitecustomize.py to be importable for test coverage purposes
        SITECUSTOMIZE_DIR = r'{sitecustomize_dir}'
        PYTHONPATH = os.environ.get('PYTHONPATH') or None
        if PYTHONPATH is None:
            PYTHONPATH_ENV_VAR = SITECUSTOMIZE_DIR
        else:
            PYTHON_PATH_ENTRIES = PYTHONPATH.split(os.pathsep)
            if SITECUSTOMIZE_DIR in PYTHON_PATH_ENTRIES:
                PYTHON_PATH_ENTRIES.remove(SITECUSTOMIZE_DIR)
            PYTHON_PATH_ENTRIES.insert(0, SITECUSTOMIZE_DIR)
            PYTHONPATH_ENV_VAR = os.pathsep.join(PYTHON_PATH_ENTRIES)
        os.environ[str('PYTHONPATH')] = str(PYTHONPATH_ENV_VAR)
        if SITECUSTOMIZE_DIR in sys.path:
            sys.path.remove(SITECUSTOMIZE_DIR)
        sys.path.insert(0, SITECUSTOMIZE_DIR)
        '''
    )
}


def generate_script(bin_dir,
                    script_name,
                    executable,
                    code_dir,
                    extra_code=None,
                    inject_coverage=False,
                    inject_sitecustomize=False):
    '''
    Generate script
    '''
    # Late import
    import pytestsalt.utils.compat as compat

    if not os.path.isdir(bin_dir):
        os.makedirs(bin_dir)

    cli_script_name = 'cli_{}.py'.format(script_name.replace('-', '_'))
    script_path = os.path.join(bin_dir, cli_script_name)

    if not os.path.isfile(script_path):
        log.info('Generating %s', script_path)

        with compat.fopen(script_path, 'w') as sfh:
            script_template = SCRIPT_TEMPLATES.get(script_name, None)
            if script_template is None:
                script_template = SCRIPT_TEMPLATES.get('common', None)
            if script_template is None:
                raise RuntimeError(
                    'Pytest Salt\'s does not know how to handle the {} script'.format(
                        script_name
                    )
                )

            if len(executable) > 128:
                # Too long for a shebang, let's use /usr/bin/env and hope
                # the right python is picked up
                executable = '/usr/bin/env python'

            script_contents = textwrap.dedent(
                '''
                #!{executable}

                from __future__ import absolute_import
                import os
                import sys

                CODE_DIR = r'{code_dir}'
                if CODE_DIR in sys.path:
                    sys.path.remove(CODE_DIR)
                sys.path.insert(0, CODE_DIR)
                '''.format(
                    executable=executable,
                    code_dir=code_dir,
                )
            )

            if extra_code:
                script_contents += '\n' + extra_code + '\n'

            if inject_coverage:
                script_contents += '\n' + SCRIPT_TEMPLATES['coverage'] + '\n'

            if inject_sitecustomize:
                script_contents += '\n{}\n'.format(
                    SCRIPT_TEMPLATES['sitecustomize'].format(
                        sitecustomize_dir=os.path.join(
                            ROOT_DIR, 'salt', 'coverage'
                        )
                    )
                )

            script_contents += '\n' + script_template.format(script_name.replace('salt-', ''))
            sfh.write(script_contents.strip())
            log.debug(
                'Wrote the following contents to temp script %s:\n%s',
                script_path,
                script_contents
            )
        fst = os.stat(script_path)
        os.chmod(script_path, fst.st_mode | stat.S_IEXEC)

    log.info('Returning script path %r', script_path)
    return script_path



# -*- coding: utf-8 -*-
'''
pytestsalt.fixtures.dirs
~~~~~~~~~~~~~~~~~~~~~~~~

pytest salt directories related fixtures
'''
# pylint: disable=redefined-outer-name

# Import Python libs
from __future__ import absolute_import
import logging
import os
import stat

# Import 3rd-party libs
import pytest

# Import pytest-salt libs
import pytestsalt.salt.engines
import pytestsalt.salt.log_handlers


log = logging.getLogger(__name__)

ROOT_DIR = 'root'
MOM_ROOT_DIR = 'mom-root'
SECONDARY_ROOT_DIR = 'secondary-root'
SESSION_ROOT_DIR = 'session-root'
SESSION_MOM_ROOT_DIR = 'session-mom-root'
SESSION_SECONDARY_ROOT_DIR = 'session-secondary-root'


@pytest.fixture
def root_dir(tempdir):
    '''
    Return the function scoped salt root dir
    '''
    return tempdir.mkdir(ROOT_DIR)


@pytest.fixture(scope='session')
def session_root_dir(tempdir):
    '''
    Return the session scoped salt root dir
    '''
    return tempdir.mkdir(SESSION_ROOT_DIR)


@pytest.fixture
def master_of_masters_root_dir(tempdir):
    '''
    Return the function scoped salt master of masters root dir
    '''
    return tempdir.mkdir(MOM_ROOT_DIR)


@pytest.fixture(scope='session')
def session_master_of_masters_root_dir(tempdir):
    '''
    Return the session scoped salt master of masters root dir
    '''
    return tempdir.mkdir(SESSION_MOM_ROOT_DIR)


@pytest.fixture
def secondary_root_dir(tempdir):
    '''
    Return the function scoped salt secondary root dir
    '''
    return tempdir.mkdir(SECONDARY_ROOT_DIR)


@pytest.fixture(scope='session')
def session_secondary_root_dir(tempdir):
    '''
    Return the session scoped salt secondary root dir
    '''
    return tempdir.mkdir(SESSION_SECONDARY_ROOT_DIR)


@pytest.fixture
def conf_dir(root_dir):
    '''
    Fixture which returns the salt configuration directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = root_dir.join('conf')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture(scope='session')
def session_conf_dir(session_root_dir):
    '''
    Fixture which returns the salt configuration directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = session_root_dir.join('conf')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture
def master_of_masters_conf_dir(master_of_masters_conf_dir):
    '''
    Fixture which returns the salt configuration directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = master_of_masters_root_dir.join('conf')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture(scope='session')
def session_master_of_masters_conf_dir(session_master_of_masters_root_dir):
    '''
    Fixture which returns the salt configuration directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = session_master_of_masters_root_dir.join('conf')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture
def secondary_conf_dir(secondary_root_dir):
    '''
    Fixture which returns the salt secondary configuration directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = secondary_root_dir.join('conf')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture(scope='session')
def session_secondary_conf_dir(session_secondary_root_dir):
    '''
    Fixture which returns the salt secondary configuration directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = session_secondary_root_dir.join('conf')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture
def syndic_conf_dir(root_dir):
    '''
    Fixture which returns the salt syndic configuration directory path.
    Creates the directory if it does not yet exist.

    Even though the salt syndic will read from both the master and minion
    configuration files, we'll store copies on this directory for complete
    separation, ie, to don't include syndic config options in either of the
    master and minion configuration files.
    '''
    dirname = root_dir.join('syndic-conf')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture(scope='session')
def session_syndic_conf_dir(session_root_dir):
    '''
    Fixture which returns the salt syndic configuration directory path.
    Creates the directory if it does not yet exist.

    Even though the salt syndic will read from both the master and minion
    configuration files, we'll store copies on this directory for complete
    separation, ie, to don't include syndic config options in either of the
    master and minion configuration files.
    '''
    dirname = session_root_dir.join('syndic-conf')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture
def master_config_includes_dir(conf_dir):
    '''
    Fixture which returns the salt master configuration includes directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = conf_dir.join('master.d')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture(scope='session')
def session_master_config_includes_dir(session_conf_dir):
    '''
    Fixture which returns the salt master configuration includes directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = session_conf_dir.join('master.d')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture
def master_of_masters_config_includes_dir(master_of_masters_conf_dir):
    '''
    Fixture which returns the salt master configuration includes directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = master_of_masters_conf_dir.join('master.d')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture(scope='session')
def session_master_of_masters_config_includes_dir(session_master_of_masters_conf_dir):
    '''
    Fixture which returns the salt master configuration includes directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = session_master_of_masters_conf_dir.join('master.d')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture
def minion_config_includes_dir(conf_dir):
    '''
    Fixture which returns the salt minion configuration includes directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = conf_dir.join('minion.d')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture(scope='session')
def session_minion_config_includes_dir(session_conf_dir):
    '''
    Fixture which returns the salt minion configuration includes directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = session_conf_dir.join('minion.d')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture
def secondary_minion_config_includes_dir(secondary_conf_dir):
    '''
    Fixture which returns the salt secondary minion configuration includes directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = secondary_conf_dir.join('minion.d')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture(scope='session')
def session_secondary_minion_config_includes_dir(session_secondary_conf_dir):
    '''
    Fixture which returns the salt secondary minion configuration includes directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = session_secondary_conf_dir.join('minion.d')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture
def integration_files_dir(root_dir):
    '''
    Fixture which returns the salt integration files directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = root_dir.join('integration-files')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture
def state_tree_root_dir(integration_files_dir):
    '''
    Fixture which returns the salt state tree root directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = integration_files_dir.join('state-tree')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture
def pillar_tree_root_dir(integration_files_dir):
    '''
    Fixture which returns the salt pillar tree root directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = integration_files_dir.join('pillar-tree')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture
def base_env_state_tree_root_dir(state_tree_root_dir):
    '''
    Fixture which returns the salt base environment state tree directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = state_tree_root_dir.join('base')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture
def prod_env_state_tree_root_dir(state_tree_root_dir):
    '''
    Fixture which returns the salt prod environment state tree directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = state_tree_root_dir.join('prod')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture
def base_env_pillar_tree_root_dir(pillar_tree_root_dir):
    '''
    Fixture which returns the salt base environment pillar tree directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = pillar_tree_root_dir.join('base')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture
def prod_env_pillar_tree_root_dir(pillar_tree_root_dir):
    '''
    Fixture which returns the salt prod environment pillar tree directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = pillar_tree_root_dir.join('prod')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture
def master_of_masters_integration_files_dir(master_of_masters_root_dir):
    '''
    Fixture which returns the salt integration files directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = master_of_masters_root_dir.join('integration-files')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture
def master_of_masters_state_tree_root_dir(master_of_masters_integration_files_dir):
    '''
    Fixture which returns the salt state tree root directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = master_of_masters_integration_files_dir.join('state-tree')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture
def master_of_masters_pillar_tree_root_dir(master_of_masters_integration_files_dir):
    '''
    Fixture which returns the salt pillar tree root directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = master_of_masters_integration_files_dir.join('pillar-tree')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture
def master_of_masters_base_env_state_tree_root_dir(master_of_masters_state_tree_root_dir):
    '''
    Fixture which returns the salt base environment state tree directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = master_of_masters_state_tree_root_dir.join('base')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture
def master_of_masters_prod_env_state_tree_root_dir(master_of_masters_state_tree_root_dir):
    '''
    Fixture which returns the salt prod environment state tree directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = master_of_masters_state_tree_root_dir.join('prod')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture
def master_of_masters_base_env_pillar_tree_root_dir(master_of_masters_pillar_tree_root_dir):
    '''
    Fixture which returns the salt base environment pillar tree directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = master_of_masters_pillar_tree_root_dir.join('base')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture
def master_of_masters_prod_env_pillar_tree_root_dir(master_of_masters_pillar_tree_root_dir):
    '''
    Fixture which returns the salt prod environment pillar tree directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = master_of_masters_pillar_tree_root_dir.join('prod')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture(scope='session')
def session_integration_files_dir(session_root_dir):
    '''
    Fixture which returns the salt integration files directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = session_root_dir.join('integration-files')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture(scope='session')
def session_state_tree_root_dir(session_integration_files_dir):
    '''
    Fixture which returns the salt state tree root directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = session_integration_files_dir.join('state-tree')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture(scope='session')
def session_pillar_tree_root_dir(session_integration_files_dir):
    '''
    Fixture which returns the salt pillar tree root directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = session_integration_files_dir.join('pillar-tree')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture(scope='session')
def session_base_env_state_tree_root_dir(session_state_tree_root_dir):
    '''
    Fixture which returns the salt base environment state tree directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = session_state_tree_root_dir.join('base')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture(scope='session')
def session_prod_env_state_tree_root_dir(session_state_tree_root_dir):
    '''
    Fixture which returns the salt prod environment state tree directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = session_state_tree_root_dir.join('prod')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture(scope='session')
def session_base_env_pillar_tree_root_dir(session_pillar_tree_root_dir):
    '''
    Fixture which returns the salt base environment pillar tree directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = session_pillar_tree_root_dir.join('base')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture(scope='session')
def session_prod_env_pillar_tree_root_dir(session_pillar_tree_root_dir):
    '''
    Fixture which returns the salt prod environment pillar tree directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = session_pillar_tree_root_dir.join('prod')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture(scope='session')
def session_master_of_masters_integration_files_dir(session_master_of_masters_root_dir):
    '''
    Fixture which returns the salt integration files directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = session_master_of_masters_root_dir.join('integration-files')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture(scope='session')
def session_master_of_masters_state_tree_root_dir(session_master_of_masters_integration_files_dir):
    '''
    Fixture which returns the salt state tree root directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = session_master_of_masters_integration_files_dir.join('state-tree')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture(scope='session')
def session_master_of_masters_pillar_tree_root_dir(session_master_of_masters_integration_files_dir):
    '''
    Fixture which returns the salt pillar tree root directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = session_master_of_masters_integration_files_dir.join('pillar-tree')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture(scope='session')
def session_master_of_masters_base_env_state_tree_root_dir(session_master_of_masters_state_tree_root_dir):
    '''
    Fixture which returns the salt base environment state tree directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = session_master_of_masters_state_tree_root_dir.join('base')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture(scope='session')
def session_master_of_masters_prod_env_state_tree_root_dir(session_master_of_masters_state_tree_root_dir):
    '''
    Fixture which returns the salt prod environment state tree directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = session_master_of_masters_state_tree_root_dir.join('prod')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture(scope='session')
def session_master_of_masters_base_env_pillar_tree_root_dir(session_master_of_masters_pillar_tree_root_dir):
    '''
    Fixture which returns the salt base environment pillar tree directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = session_master_of_masters_pillar_tree_root_dir.join('base')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture(scope='session')
def session_master_of_masters_prod_env_pillar_tree_root_dir(session_master_of_masters_pillar_tree_root_dir):
    '''
    Fixture which returns the salt prod environment pillar tree directory path.
    Creates the directory if it does not yet exist.
    '''
    dirname = session_pillar_tree_root_dir.join('prod')
    dirname.ensure(dir=True)
    return dirname


@pytest.fixture
def sshd_config_dir(tempdir):
    '''
    Return the path to a configuration directory for the sshd server
    '''
    config_dir = tempdir.join('sshd')
    config_dir.ensure(dir=True)
    return config_dir


@pytest.fixture(scope='session')
def session_sshd_config_dir(tempdir):
    '''
    Return the path to a configuration directory for a session scoped sshd server
    '''
    config_dir = tempdir.join('session-sshd')
    config_dir.ensure(dir=True)
    return config_dir


@pytest.fixture
def sshd_priv_dir():
    '''
    Create the privdir for starting sshd on systems that do not have an sshd
    daemon already running
    '''
    if not os.path.isdir('/var/run/sshd'):
        os.mkdir('/var/run/sshd/')
        os.chmod('/var/run/sshd/', stat.S_IRWXU | stat.S_IRGRP | stat.S_IXGRP | stat.S_IROTH | stat.S_IXOTH)
        yield
        os.rmdir('/var/run/sshd')


@pytest.fixture(scope='session')
def session_sshd_priv_dir():
    '''
    Create the privdir for starting sshd on systems that do not have an sshd
    daemon already running
    '''
    if not os.path.isdir('/var/run/sshd'):
        os.mkdir('/var/run/sshd/')
        os.chmod('/var/run/sshd/', stat.S_IRWXU | stat.S_IRGRP | stat.S_IXGRP | stat.S_IROTH | stat.S_IXOTH)
        yield
        os.rmdir('/var/run/sshd')


@pytest.fixture
def ssh_config_dir(tempdir):
    '''
    Return the path to a configuration directory for the ssh client
    '''
    config_dir = tempdir.join('ssh-client')
    config_dir.ensure(dir=True)
    return config_dir


@pytest.fixture(scope='session')
def session_ssh_config_dir(tempdir):
    '''
    Return the path to a configuration directory for a session scoped ssh client
    '''
    config_dir = tempdir.join('session-ssh-client')
    config_dir.ensure(dir=True)
    return config_dir


@pytest.fixture(scope='session')
def log_handlers_dir():
    '''
    Return the directory name for the pytest-salt log handlers directory
    '''
    return os.path.dirname(pytestsalt.salt.log_handlers.__file__)


@pytest.fixture(scope='session')
def engines_dir():
    '''
    Return the directory name for the pytest-salt engines directory
    '''
    return os.path.dirname(pytestsalt.salt.engines.__file__)



# -*- coding: utf-8 -*-
'''
pytestsalt.utils.compat
~~~~~~~~~~~~~~~~~~~~~~~

Imports compatability layer
'''

try:
    # Salt > 2017.1.1
    # pylint: disable=invalid-name
    import salt.utils.files
    fopen = salt.utils.files.fopen
except AttributeError:
    # Salt <= 2017.1.1
    # pylint: disable=invalid-name
    fopen = salt.utils.fopen

try:
    # Salt >= 2018.3.0
    # pylint: disable=invalid-name
    from salt.utils.path import which
except ImportError:
    # Salt < 2018.3.0
    # pylint: disable=invalid-name
    from salt.utils import which




# This file was generated by 'versioneer.py' (0.18) from
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

from __future__ import absolute_import
import json

version_json = '''
{
 "date": "2019-08-15T02:58:37+0100",
 "dirty": false,
 "error": null,
 "full-revisionid": "eda138b001a07f5afb2a971db2fe2f6baba602a6",
 "version": "2019.8.16"
}
'''  # END VERSION_JSON


def get_versions():
    return json.loads(version_json)
